
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
// ARC algorithm interace declarations
//

/*
 *  This header defines all types, constants, and functions used by
 *  applications that use the ARC (Asynchronous sample-Rate Conversion) algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  the ability to incorporate multiple implementations of the ARC
 *  algorithm in a single application at the expense of some
 *  additional indirection.
 */

#ifndef ARC_
#define ARC_

#include <alg.h>
#include <ialg.h>
#include <paf_alg.h>
#include <iarc.h>

#include "paftyp.h"

/*
 *  ======== ARC_Handle ========
 *  ARC (Asynchronous sample-Rate Conversion) algorithm instance handle
 */
typedef struct IARC_Obj *ARC_Handle;

/*
 *  ======== ARC_Params ========
 *  ARC (Asynchronous sample-Rate Conversion) algorithm instance creation parameters
 */
typedef struct IARC_Params ARC_Params;

/*
 *  ======== ARC_PARAMS ========
 *  Default instance parameters
 */
#define ARC_PARAMS IARC_PARAMS

/*
 *  ======== ARC_Status ========
 *  Status structure for getting ARC instance attributes
 */
typedef volatile struct IARC_Status ARC_Status;

/*
 *  ======== ARC_Cmd ========
 *  This typedef defines the control commands ARC objects
 */
typedef IARC_Cmd   ARC_Cmd;

/*
 * ===== control method commands =====
 */
#define ARC_NULL IARC_NULL
#define ARC_GETSTATUSADDRESS1 IARC_GETSTATUSADDRESS1
#define ARC_GETSTATUSADDRESS2 IARC_GETSTATUSADDRESS2
#define ARC_GETSTATUS IARC_GETSTATUS
#define ARC_SETSTATUS IARC_SETSTATUS

/*
 *  ======== ARC_create ========
 *  Create an instance of a ARC object.
 */
static inline ARC_Handle ARC_create(const IARC_Fxns *fxns, const ARC_Params *prms)
{
    return ((ARC_Handle)PAF_ALG_create((IALG_Fxns *)fxns, NULL, (IALG_Params *)prms,NULL,NULL));
}

/*
 *  ======== ARC_delete ========
 *  Delete a ARC instance object
 */
static inline Void ARC_delete(ARC_Handle handle)
{
    PAF_ALG_delete((ALG_Handle)handle);
}

/*
 *  ======== ARC_apply ========
 */
extern Int ARC_apply(ARC_Handle, PAF_AudioFrame *);

/*
 *  ======== ARC_reset ========
 */
extern Int ARC_reset(ARC_Handle, PAF_AudioFrame *);

/*
 *  ======== ARC_exit ========
 *  Module finalization
 */
extern Void ARC_exit(Void);

/*
 *  ======== ARC_init ========
 *  Module initialization
 */
extern Void ARC_init(Void);

#endif  /* ARC_ */
