
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
// ARC algorithm interface delarations
//

/*
 *  Vendor specific (TIH) interface header for
 *  ARC (Asynchronous sample-Rate Conversion) algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  and minimal overhead at the expense of being tied to a
 *  particular ARC implementation.
 *
 *  This header only contains declarations that are specific
 *  to this implementation.  Thus, applications that do not
 *  want to be tied to a particular implementation should never
 *  include this header (i.e., it should never directly
 *  reference anything defined in this header.)
 */

#ifndef ARC_TIH_
#define ARC_TIH_

#include <ialg.h>

#include <iarc.h>
#include <icom.h>

/*
 *  ======== ARC_TIH_exit ========
 *  Required module finalization function
 */
#define ARC_TIH_exit COM_TII_exit

/*
 *  ======== ARC_TIH_init ========
 *  Required module initialization function
 */
#define ARC_TIH_init COM_TII_init

/*
 *  ======== ARC_TIH_IALG ========
 *  TIH's implementation of ARC's IALG interface
 */
extern IALG_Fxns ARC_TIH_IALG; 

/*
 *  ======== ARC_TIH_IARC ========
 *  TIH's implementation of ARC's IARC interface
 */
extern const IARC_Fxns ARC_TIH_IARC; 


/*
 *  ======== Vendor specific methods  ========
 *  The remainder of this file illustrates how a vendor can
 *  extend an interface with custom operations.
 *
 *  The operations below simply provide a type safe interface 
 *  for the creation, deletion, and application of TIH's
 *  ARC (Asynchronous sample-Rate Conversion) algorithm. However, other
 *  implementation specific operations can also be added.
 */

/*
 *  ======== ARC_TIH_Handle ========
 */
typedef struct ARC_TIH_Obj *ARC_TIH_Handle;

/*
 *  ======== ARC_TIH_Params ========
 *  We don't add any new parameters to the standard ones defined by IARC.
 */
typedef IARC_Params ARC_TIH_Params;

/*
 *  ======== ARC_TIH_Status ========
 *  We don't add any new status to the standard one defined by IARC.
 */
typedef IARC_Status ARC_TIH_Status;

/*
 *  ======== ARC_TIH_Config ========
 *  We don't add any new config to the standard one defined by IARC.
 */
typedef IARC_Config ARC_TIH_Config;

/*
 *  ======== ARC_TIH_PARAMS ========
 *  Define our default parameters.
 */
#define ARC_TIH_PARAMS   IARC_PARAMS

/*
 *  ======== ARC_TIH_create ========
 *  Create a ARC_TIH instance object.
 */
extern ARC_TIH_Handle ARC_TIH_create(const ARC_TIH_Params *params);

/*
 *  ======== ARC_TIH_delete ========
 *  Delete a ARC_TIH instance object.
 */
#define ARC_TIH_delete COM_TII_delete

#endif  /* ARC_TIH_ */
