
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Standard Deemphasis Algorithm interface declarations
//
//
//

/*
 *  This header defines all types, constants, and functions used by 
 *  applications that use the Deemphasis Algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  the ability to incorporate multiple implementations of the DEM
 *  algorithm in a single application at the expense of some
 *  additional indirection.
 */
#ifndef DEM_
#define DEM_

#include <alg.h>
#include <ialg.h>
#include <paf_alg.h>

#include "idem.h"
#include "paftyp.h"

/*
 *  ======== DEM_Handle ========
 *  Deemphasis Algorithm instance handle
 */
typedef struct IDEM_Obj *DEM_Handle;

/*
 *  ======== DEM_Params ========
 *  Deemphasis Algorithm instance creation parameters
 */
typedef struct IDEM_Params DEM_Params;

/*
 *  ======== DEM_PARAMS ========
 *  Default instance parameters
 */
#define DEM_PARAMS IDEM_PARAMS

/*
 *  ======== DEM_Status ========
 *  Status structure for getting DEM instance attributes
 */
typedef volatile struct IDEM_Status DEM_Status;

/*
 *  ======== DEM_Cmd ========
 *  This typedef defines the control commands DEM objects
 */
typedef IDEM_Cmd DEM_Cmd;

/*
 * ===== control method commands =====
 */
#define DEM_NULL IDEM_NULL
#define DEM_GETSTATUSADDRESS1 IDEM_GETSTATUSADDRESS1
#define DEM_GETSTATUSADDRESS2 IDEM_GETSTATUSADDRESS2
#define DEM_GETSTATUSFIL IDEM_GETSTATUSFIL
#define DEM_GETSTATUS IDEM_GETSTATUS
#define DEM_SETSTATUS IDEM_SETSTATUS

/*
 *  ======== DEM_create ========
 *  Create an instance of a DEM object.
 */
static inline DEM_Handle DEM_create(const IDEM_Fxns *fxns, const DEM_Params *prms)
{
    return ((DEM_Handle)PAF_ALG_create((IALG_Fxns *)fxns, NULL, (IALG_Params *)prms,NULL,NULL));
}

/*
 *  ======== DEM_delete ========
 *  Delete a DEM instance object
 */
static inline Void DEM_delete(DEM_Handle handle)
{
    PAF_ALG_delete((ALG_Handle)handle);
}

/*
 *  ======== DEM_apply ========
 */
extern Int DEM_apply(DEM_Handle, PAF_AudioFrame *);

/*
 *  ======== DEM_reset ========
 */
extern Int DEM_reset(DEM_Handle, PAF_AudioFrame *);

/*
 *  ======== DEM_exit ========
 *  Module finalization
 */
extern Void DEM_exit(Void);

/*
 *  ======== DEM_init ========
 *  Module initialization
 */
extern Void DEM_init(Void);

#endif  /* DEM_ */

