
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Common Deemphasis Algorithm interface declarations
//
//
//

/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the Deemphasis Algorithm.
 */
#ifndef IDEM_
#define IDEM_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>

#include "ifil.h"
#include "fil.h"

#include "icom.h"
#include "paftyp.h"

/*
 *  ======== IDEM_Obj ========
 *  Every implementation of IDEM *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct IDEM_Obj {
    struct IDEM_Fxns *fxns;    /* function list: standard, public, private */
} IDEM_Obj;

/*
 *  ======== IDEM_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct IDEM_Obj *IDEM_Handle;

/*
 *  ======== IDEM_Status ========
 *  This Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.
 */
typedef volatile struct IDEM_Status {
    Int size;
    XDAS_Int8 mode;
    XDAS_Int8 filterMode;
    XDAS_Int8 filterActive;
    XDAS_Int8 unused1;
#if PAF_MAXNUMCHAN <= 8     /* PAF_ChannelMask = XDAS_Int8 */
    PAF_ChannelMask channelEnable;
    PAF_ChannelMask channelActive;
    XDAS_Int8 unused2[2];
#elif PAF_MAXNUMCHAN <= 16  /* PAF_ChannelMask = XDAS_Int16 */
    PAF_ChannelMask channelEnable;
    PAF_ChannelMask channelActive;
#elif PAF_MAXNUMCHAN <= 32  /* PAF_ChannelMask = XDAS_Int32 */
    PAF_ChannelMask channelEnable;
    PAF_ChannelMask channelActive;
#else
#error unsupported option: PAF_MAXNUMCHAN > 32
#endif
} IDEM_Status;

/*
 *  ======== IDEM_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct IDEM_Config {
    Int size;
    XDAS_Int8 filterActivePrev;
} IDEM_Config;

/*
 *  ======== IDEM_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a DEM object.
 *
 *  Every implementation of IDEM *must* declare this structure as
 *  the first member of the implementation's parameter structure.
 */

typedef struct IDEM_Active {
    Int size;
    Int flag;
    IFIL_Fxns *ifil_Fxns;
}IDEM_Active;

extern const IDEM_Active IDEM_ACTIVE;

typedef struct IDEM_Params {
    Int size;
    const IDEM_Status *pStatus;
    const IDEM_Config *pConfig;
    const IDEM_Active *pActive;
    const IFIL_Params *pParams;
} IDEM_Params;

/*
 *  ======== IDEM_PARAMS ========
 *  Default instance creation parameters (defined in idem.c)
 */
extern const IDEM_Params IDEM_PARAMS;

/*
 *  ======== IDEM_Fxns ========
 *  All implementation's of DEM must declare and statically 
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is DEM_<vendor>_IDEM, where
 *  <vendor> is the vendor name.
 */
typedef struct IDEM_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int         (*reset)(IDEM_Handle, PAF_AudioFrame *);
    Int         (*apply)(IDEM_Handle, PAF_AudioFrame *);
    /* private */
} IDEM_Fxns;

/*
 *  ======== IDEM_Cmd ========
 *  The Cmd enumeration defines the control commands for the DEM
 *  control method.
 */
typedef enum IDEM_Cmd {
    IDEM_NULL                   = ICOM_NULL,
    IDEM_GETSTATUSADDRESS1      = ICOM_GETSTATUSADDRESS1,
    IDEM_GETSTATUSADDRESS2      = ICOM_GETSTATUSADDRESS2,
    IDEM_GETSTATUS              = ICOM_GETSTATUS,
    IDEM_SETSTATUS              = ICOM_SETSTATUS,
    IDEM_GETSTATUSFIL
} IDEM_Cmd;

#endif  /* IDEM_ */

