
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// MTX algorithm interface declarations.
//
//
//

/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the ASP Example Demonstration algorithm.
 */
#ifndef IMTX_
#define IMTX_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>

#include "icom.h"
#include "paftyp.h"

#include "WaTypes.h"
#include <cpl.h>

#define TILIB_DELAY  1
#define TILIB_BUFFER 1

#if TILIB_DELAY

#if TILIB_BUFFER
#define NUM_MTX_MemRec 7
#else
#define NUM_MTX_MemRec 6
#endif  //buffer

#else
#define NUM_MTX_MemRec 5

#endif  //delay


/*
 *  ======== IMTX_Obj ========
 *  Every implementation of IMTX *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct IMTX_Obj {
    struct IMTX_Fxns *fxns;    /* function list: standard, public, private */
} IMTX_Obj;

/*
 *  ======== IMTX_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct IMTX_Obj *IMTX_Handle;

/*
 *  ======== IMTX_Status ========
 *  This Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.  This structure is actually
 *  instantiated and initialized in imtx.c.
 */
typedef volatile struct IMTX_Status {

    Int size;              /* This value must always be here, and must be set to the 
                             total size of this structure in 8-bit bytes, as the 
                             sizeof() operator would do. */
    XDAS_Int8 mode;        /* This is the 8-bit MTX Mode Control Register.  All 
                             Algorithms must have a mode control register.  */
    XDAS_Int8 use;         /* This is the 8-bit MTX Use Control Register. */
    XDAS_Int16 Preset; /* load a new preset if != 0 */
    XDAS_Int16 CtrMix; /* Scale L+R to the center,        0x4000 = -3dB */
    XDAS_Int16 SubMix; /* Scale L+R to the Subwoofer,     0x4000 = -3dB */
    // typically either wet or dry will be selected...
    XDAS_Int16 SurrDry; /* Scale Dry Input to sidesurr,   0x4000 = x1   */
    XDAS_Int16 SurrWet; /* Scale Delayed L-R to sidesurr, 0x4000 = x1 */
    XDAS_Int16 BackDry; /* Scale Dry Input to backsurr,   0x4000 = x1  */
    XDAS_Int16 BackWet; /* Scale Delayed L-R to backsurr, 0x4000 = x1  */
    // Scale the surrounds pairwise...
    XDAS_Int16 SurrGain; /* Scale Side Surround Pair,     0x4000 = x1  */
    XDAS_Int16 BackGain; /* Scale Back Surround Pair,     0x4000 = x1  */
    XDAS_Int16 surrDelay; /* typically left at 42msec */
    
} IMTX_Status;

typedef struct MtxPreset{
   int16 Preset;
   int16 CtrMix;
   int16 SubMix;
   int16 SurrDry;
   int16 SurrWet;
   int16 BackDry;
   int16 BackWet;
   int16 SurrGain;
   int16 BackGain;
   int16 surrDelay;
} MtxPreset;

typedef struct MtxParam{
//    int32   chCfg;    /* channel config */
//    float32 fs;       /* sampling rate  */

    float32 centerMix;     /* Scale L+R to the center,        0x4000 = -3dB     */
    float32 subwoofMix;    /* Scale L+R to the Subwoofer,     0x4000 = -3dB     */
    // typically either wet or dry will be selected...
    float32 surroundDry;    /* Scale Dry Input to sidesurr,    0x4000 = x1       */
    float32 surroundWet;    /* Scale Delayed L-R to sidesurr,  0x4000 = x1       */
    float32 backsurrDry;    /* Scale Dry Input to backsurr,    0x4000 = x1       */
    float32 backsurrWet;    /* Scale Delayed L-R to backsurr,  0x4000 = x1       */
    // Scale the surrounds pairwise...
    float32 surroundGain;   /* Scale Side Surround Pair,       0x4000 = x1       */
    float32 backsurrGain;   /* Scale Back Surround Pair,       0x4000 = x1       */

    float32 sdelay;   /* surrdelay in fractional sec */

} MtxParam;

typedef struct {
    /* These two are for activate/deactivate */
    Int size;
    Void *base;

    int32 flags;        /* flags, if any */
    MtxParam *param;    /* parameters */
    WaDelay *delay;
    float *tmpbuf;
} MatrixObj;


/*
 *  ======== IMTX_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct IMTX_Config {

    MatrixObj *matrix;
    XDAS_Int16 lastSampRate;    /* last sampling rate from pAudioFrame        */
    XDAS_Int16 lastSatConfig;   /* last satellite config from pAudioFrame     */
    XDAS_Int16 currSatConfig;   /* updated parameter of satellite config      */
    XDAS_Int16 resetFlag;       /* indicate reset condition arising out of activation sequence */
    float PrevSurdelay;
    Void *pMems[NUM_MTX_MemRec];
} IMTX_Config;

/*
 *  ======== IMTX_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a MTX object.
 *
 *  Every implementation of IMTX *must* declare this structure as
 *  the first member of the implementation's parameter structure.  This structure is actually
 *  instantiated and initialized in imtx.c.
 */
typedef struct IMTX_Params {
    Int size;
    const IMTX_Status *pStatus;
    IMTX_Config config;
} IMTX_Params;


/*
 *  ======== IMTX_PARAMS ========
 *  Default instance creation parameters (defined in imtx.c)
 */
extern const IMTX_Params IMTX_PARAMS;

/*
 *  ======== IMTX_Fxns ========
 *  All implementation's of MTX must declare and statically 
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is MTX_<vendor>_IMTX, where
 *  <vendor> is the vendor name.
 */
typedef struct IMTX_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int         (*reset)(IMTX_Handle, PAF_AudioFrame *);
    Int         (*apply)(IMTX_Handle, PAF_AudioFrame *);

    /* private */
    Void    (*WaMtxLoadPreset)(IMTX_Handle);
    Void    (*MtxSetParams)(IMTX_Handle);
} IMTX_Fxns;

/*
 *  ======== IMTX_Cmd ========
 *  The Cmd enumeration defines the control commands for the MTX
 *  control method.
 */
typedef enum IMTX_Cmd {
    IMTX_NULL                    = ICOM_NULL,
    IMTX_GETSTATUSADDRESS1       = ICOM_GETSTATUSADDRESS1,
    IMTX_GETSTATUSADDRESS2       = ICOM_GETSTATUSADDRESS2,
    IMTX_GETSTATUS               = ICOM_GETSTATUS,
    IMTX_SETSTATUS               = ICOM_SETSTATUS
} IMTX_Cmd;

#endif  /* IMTX_ */

