
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// MTX interface declarations
//
//
//

/*
 *  Vendor specific (WAV) interface header for
 *  ASP Example Demonstration algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  and minimal overhead at the expense of being tied to a
 *  particular MTX implementation.
 *
 *  This header only contains declarations that are specific
 *  to this implementation.  Thus, applications that do not
 *  want to be tied to a particular implementation should never
 *  include this header (i.e., it should never directly
 *  reference anything defined in this header.)
 */
#ifndef MTX_WAV_
#define MTX_WAV_

#include <ialg.h>

#include <imtx.h>
#include <icom.h>

/*
 *  ======== MTX_WAV_exit ========
 *  Required module finalization function
 */
#define MTX_WAV_exit COM_TII_exit

/*
 *  ======== MTX_WAV_init ========
 *  Required module initialization function
 */
#define MTX_WAV_init COM_TII_init

/*
 *  ======== MTX_WAV_IALG ========
 *  WAV's implementation of MTX's IALG interface
 */
extern IALG_Fxns MTX_WAV_IALG; 

/*
 *  ======== MTX_WAV_IMTX ========
 *  WAV's implementation of MTX's IMTX interface
 */
extern const IMTX_Fxns MTX_WAV_IMTX; 


/*
 *  ======== Vendor specific methods  ========
 *  The remainder of this file illustrates how a vendor can
 *  extend an interface with custom operations.
 *
 *  The operations below simply provide a type safe interface 
 *  for the creation, deletion, and application of WAV's
 *  ASP Example Demonstration algorithm. However, other
 *  implementation specific operations can also be added.
 */

/*
 *  ======== MTX_WAV_Handle ========
 */
typedef struct MTX_WAV_Obj *MTX_WAV_Handle;

/*
 *  ======== MTX_WAV_Params ========
 *  We don't add any new parameters to the standard ones defined by IMTX.
 */
typedef IMTX_Params MTX_WAV_Params;

/*
 *  ======== MTX_WAV_Status ========
 *  We don't add any new status to the standard one defined by IMTX.
 */
typedef IMTX_Status MTX_WAV_Status;

/*
 *  ======== MTX_WAV_Config ========
 *  We don't add any new config to the standard one defined by IMTX.
 */
typedef IMTX_Config MTX_WAV_Config;

/*
 *  ======== MTX_WAV_PARAMS ========
 *  Define our default parameters.
 */
#define MTX_WAV_PARAMS   IMTX_PARAMS

/*
 *  ======== MTX_WAV_create ========
 *  Create a MTX_WAV instance object.
 */
extern MTX_WAV_Handle MTX_WAV_create(const MTX_WAV_Params *params);

/*
 *  ======== MTX_WAV_delete ========
 *  Delete a MTX_WAV instance object.
 */
#define MTX_WAV_delete COM_TII_delete

#endif  /* MTX_WAV_ */
