
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// RVB interface declarations
//

/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the ASP Example Demonstration algorithm.
 */
#ifndef IRVB_
#define IRVB_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>

#include "icom.h"
#include "paftyp.h"

#include "WaRvb.h"

#define MAX_NUM_MEMTABS 32

#define RVB_DCONFIG_STD           0
#define RVB_DCONFIG_SURDELAY      1
#define RVB_DCONFIG_PRESURDELAY   2



/*
 *  ======== IRVB_Obj ========
 *  Every implementation of IRVB *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct IRVB_Obj {
    struct IRVB_Fxns *fxns;    /* function list: standard, public, private */
} IRVB_Obj;

/*
 *  ======== IRVB_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct IRVB_Obj *IRVB_Handle;

/*
 *  ======== IRVB_Status ========
 *  This Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.  This structure is actually
 *  instantiated and initialized in iae.c.
 */


// Values for paramChg to manipulate params and presets
#define DYN    1
#define STAT   2
#define ALL    3
#define LOAD   4
#define STORE  8


typedef volatile struct IRVB_Status {

    Int size;             /* This value must always be here, and must be set
                             to the total size of this structure in 8-bit
                             bytes, as the sizeof() operator would do. */
    XDAS_Int8 mode;       /* This is the 8-bit RVB Mode Control Register. All
                             Algorithms must have a mode control register.  */
// we may use this for preset control
    XDAS_Int8 use;        /* This is the 8-bit RVB Use register. This is
                             intended to be used to engage/disengage the RVB
                             algorithm, rather than using the Mode register
                             for that purpose. */
    XDAS_Int16 paramChg;  /* paramChg bit 0 means dynamic params changed, bit
                             1 means static params changed. parameters changed
                             if != 0 */

/* these dynamic parameters can be changed without clicks */
    XDAS_Int16 fxMix;            /* effects mix: 0 = dry, 100 = wet    */
    XDAS_Int16 reverbTime;       /* reverberation time, milliseconds */
    XDAS_Int16 damping;          /* reverberation damping frequency, Hz */

/* these static parameters can be changed but may cause clicks */
    XDAS_Int16 frontInGain;      /* L, R channel input gain, 0 to 1 scaled 2^14 */
    XDAS_Int16 centerInGain;     /* C, SW channel input gain, 0 to 1 scaled 2^14 */
    XDAS_Int16 surrInGain;       /* LS, RS, LBS, RBS channel input gain, 0 to 1 scaled 2^14 */
    XDAS_Int16 frontOutGain;     /* L, R channel output gain, 0 to 1 scaled 2^14 */
    XDAS_Int16 centerOutGain;    /* C, SW channel output gain, 0 to 1 scaled 2^14 */
    XDAS_Int16 surrOutGain;      /* LS, RS, LBS, RBS channel output gain, 0 to 1 scaled 2^14 */
    XDAS_Int16 roomSize;         /* room size, 0 = smallest, 100 = largest */
    XDAS_Int16 LFCut;            /* low frequency cut in reverb send */
    XDAS_Int16 PreDelay;         /* input predelay to rvb */
    XDAS_Int16 SDelay;           /* post delay to surrounds, 0<100msec */
    XDAS_Int16 loadPreset;       /* select from pre-compiled presets */
} IRVB_Status;


/*
 *  ======== IRVB_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct IRVB_Config {

    XDAS_Int16  delayConfig;       /* To Specify which RAM COnfig is to be selected*/
    IALG_MemAttrs  commonMem;      /* To Specify from which Common area should RVB get its RAM*/
    float32 inScale;               /* The inScale (Float to Fixed Conversion)used by Core Reverberator*/
    float32 surScale;              /* The surScale to avoid Surround Delay Saturation*/
    WaRvb *rvb;                    /* pointer to reverb object */
    XDAS_Int16 lastSampRate;       /* last sampling rate from pAudioFrame      */
    XDAS_Int16 lastNumSatStream;   /* last satellite count from pAudioFrame in */
    XDAS_Int16 lastNumSatRequest;  /* last sat count from output request       */
    XDAS_Int16 resetFlag;
    Void *pMems[MAX_NUM_MEMTABS];
} IRVB_Config;

/*
 *  ======== IRVB_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a RVB object.
 *
 *  Every implementation of IRVB *must* declare this structure as
 *  the first member of the implementation's parameter structure.  This structure is actually
 *  instantiated and initialized in iae.c.
 */
typedef struct IRVB_Params {
    Int size;
    const IRVB_Status *pStatus;
    IRVB_Config config;
} IRVB_Params;

/*
 *  ======== IRVB_PARAMS ========
 *  Default instance creation parameters (defined in iae.c)
 */
extern const IRVB_Params IRVB_PARAMS;
extern const IRVB_Params IRVB_PARAMS_STD;
extern const IRVB_Params IRVB_PARAMS_SDELAY;
extern const IRVB_Params IRVB_PARAMS_PSDELAY;

extern const IRVB_Params IRVB_PARAMS_PSDELAY_C10;

/*
 *  ======== IRVB_Fxns ========
 *  All implementation's of RVB must declare and statically
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is RVB_<vendor>_IRVB, where
 *  <vendor> is the vendor name.
 */
typedef struct IRVB_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int         (*reset)(IRVB_Handle, PAF_AudioFrame *);
    Int         (*apply)(IRVB_Handle, PAF_AudioFrame *);
    /* private */
} IRVB_Fxns;

/*
 *  ======== IRVB_Cmd ========
 *  The Cmd enumeration defines the control commands for the RVB
 *  control method.
 */
typedef enum IRVB_Cmd {
    IRVB_NULL                    = ICOM_NULL,
    IRVB_GETSTATUSADDRESS1       = ICOM_GETSTATUSADDRESS1,
    IRVB_GETSTATUSADDRESS2       = ICOM_GETSTATUSADDRESS2,
    IRVB_GETSTATUS               = ICOM_GETSTATUS,
    IRVB_SETSTATUS               = ICOM_SETSTATUS
} IRVB_Cmd;

#endif  /* IRVB_ */
