
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// RVB interface declarations


/*
 *  This header defines all types, constants, and functions used by 
 *  applications that use the ASP Example Demonstration algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  the ability to incorporate multiple implementations of the RVB
 *  algorithm in a single application at the expense of some
 *  additional indirection.
 */
#ifndef RVB_
#define RVB_

#include <alg.h>
#include <ialg.h>
#include <paf_alg.h>
#include <irvb.h>

#include "paftyp.h"

/*
 *  ======== RVB_Handle ========
 *  ASP Example Demonstration algorithm instance handle
 */
typedef struct IRVB_Obj *RVB_Handle;

/*
 *  ======== RVB_Params ========
 *  ASP Example Demonstration algorithm instance creation parameters
 */
typedef struct IRVB_Params RVB_Params;

/*
 *  ======== RVB_PARAMS ========
 *  Default instance parameters
 */
#define RVB_PARAMS IRVB_PARAMS

/*
 *  ======== RVB_Status ========
 *  Status structure for getting RVB instance attributes
 */
typedef volatile struct IRVB_Status RVB_Status;

/*
 *  ======== RVB_Cmd ========
 *  This typedef defines the control commands RVB objects
 */
typedef IRVB_Cmd   RVB_Cmd;

/*
 * ===== control method commands =====
 */
#define RVB_NULL IRVB_NULL
#define RVB_GETSTATUSADDRESS1 IRVB_GETSTATUSADDRESS1
#define RVB_GETSTATUSADDRESS2 IRVB_GETSTATUSADDRESS2
#define RVB_GETSTATUS IRVB_GETSTATUS
#define RVB_SETSTATUS IRVB_SETSTATUS

/*
 *  ======== RVB_create ========
 *  Create an instance of a RVB object.
 */
static inline RVB_Handle RVB_create(const IRVB_Fxns *fxns, const RVB_Params *prms)
{
    return ((RVB_Handle)PAF_ALG_create((IALG_Fxns *)fxns, NULL, (IALG_Params *)prms,NULL,NULL));
}

/*
 *  ======== RVB_delete ========
 *  Delete a RVB instance object
 */
static inline Void RVB_delete(RVB_Handle handle)
{
    PAF_ALG_delete((ALG_Handle)handle);
}

/*
 *  ======== RVB_apply ========
 */
extern Int RVB_apply(RVB_Handle, PAF_AudioFrame *);

/*
 *  ======== RVB_reset ========
 */
extern Int RVB_reset(RVB_Handle, PAF_AudioFrame *);

/*
 *  ======== RVB_exit ========
 *  Module finalization
 */
extern Void RVB_exit(Void);

/*
 *  ======== RVB_init ========
 *  Module initialization
 */
extern Void RVB_init(Void);

#endif  /* RVB_ */
