
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Header for room simulation effect.


#ifndef _WA_RVB_H
#define _WA_RVB_H

#include "WaSurrRvb.h"
#include "StereoDownMixer.h"
#include "DownSamp2.h"
#include "UpSamp2.h"
#include "WaSampType.h"
#include "WaTypes.h"
#include "cpl_vecops.h"

#ifdef  __cplusplus
extern "C" {
#endif

/*
 * Channel assignments for WaRvb
 */
#define kLeft       0
#define kRight      1
#define kCenter     2
#define kSubwoofer  3
#define kSurr       4   /* mono (center) surround */
#define kLeftSurr   4
#define kRightSurr  5
#define kBack       6   /* mono (center) back */
#define kLeftBack   6
#define kRightBack  7

#define WARVB_MAX_SAT_CHAN  7

#define MUNGING_PRESETS 0

/* Bit Mask for output testing */
#define LEFT_ON       1<<kLeft
#define RIGHT_ON      1<<kRight
#define CENTER_ON     1<<kCenter
#define SUBWOOFER_ON  1<<kSubwoofer
#define LEFTSURR_ON   1<<kLeftSurr
#define RIGHTSURR_ON  1<<kRightSurr
#define LEFTBACK_ON   1<<kLeftBack
#define RIGHTBACK_ON  1<<kRightBack

/*
 * Static WaRvb parameters. Changing these parameters while the
 * audio is running may causes clicks.
 */
typedef struct {
   int32 numInChan;     /* number satellite input channels */
   int32 numOutChan;    /* number satellite output channels */
   int32 mixSubIn;      /* true if mix in subwoofer input */
   float32 sampRate;    /* sampling rate, Hz */
   float32 roomSize;    /* room size: 0 = min, 1 = max */
   float32 LFCut;       /* hipass in reverb send */
   float32 PreDelay;    /* placeholder. deletable, fix block size */
   float32 SDelay;      /* post delay to surrounds, 0<100msec */
    /*
     * The input gain parameters determine how much signal gets into
     * the reverberator. For certain settings it may be useful to turn
     * down the center and/or surround gains, and only reverberate the
     * main L, R channels.
     */
    float32 frontInGain;    /* L, R channel input gain */
    float32 centerInGain;   /* C, SW channel input gain */
    float32 surrInGain;     /* LS, RS, LBS, RBS channel input gain */
    float32 frontOutGain;   /* L, R channel output gain */
    float32 centerOutGain;  /* C channel output gain */
    float32 surrOutGain;    /* LS, RS, LBS, RBS channel output gain */
    int32   loadPreset;     /* current running preset */
} WaRvbStaticParam;

/*
 * Dynamic WaRvb parameters. These parameters can be changed smoothly
 * while the audio is running.
 */
typedef struct {
   float32 fxMix;          /* effects mix: 0 = dry, 1 = wet */
   float32 reverbTime;     /* reverberation time, sec */
   float32 damping;        /* reverberation damping frequency, Hz */
} WaRvbDynParam;

#define WARVB_NUMPRESETS 18 // see rvb_wav_irvb.c

typedef struct WaRvbPreset {
   int16 loadPreset;       /* preset number */

   int16 frontInGain;      /* L, R channel input gain, 0 to 1 scaled 2^14 */
   int16 centerInGain;     /* C, SW channel input gain, 0 to 1 scaled 2^14 */
   int16 surrInGain;       /* LS, RS, LBS, RBS channel input gain, 0 to 1 scaled 2^14 */
   int16 frontOutGain;     /* L, R channel output gain, 0 to 1 scaled 2^14 */
   int16 centerOutGain;    /* C, SW channel output gain, 0 to 1 scaled 2^14 */
   int16 surrOutGain;      /* LS, RS, LBS, RBS channel output gain, 0 to 1 scaled 2^14 */

   int16 roomSize;         /* room size, 0 = smallest, 100 = largest */
   int16 LFCut;
   int16 fxMix;            /* effects mix: 0 = dry, 100 = wet    */
   int16 reverbTime;       /* reverberation time, milliseconds */
   int16 damping;          /* reverberation damping frequency, Hz */
   int16 PreDelay;         /* placeholder. deletable, fix block size */
   int16 sdelay;           /* post delay to surrounds, 0<100msec */

} WaRvbPreset;

// typedef struct WaRvbPreset WaRvbPresets[];



#define MAX_CHAN            8

#define WA_RVB_NUM_IN_TMP       2   /* number tmp in: L, R */
#define WA_RVB_NUM_OUT_TMP      6   /* number tmp out: L, R, LS, RS, [C, CS] */

/*
 * Channel indices in temp buffers for inputs and outputs.
 * WaSurrRvb is stereo -> quad. The center channels are mixed
 * as needed based on external channel configuration.
 */
#define kTmpLeft            0
#define kTmpRight           1
#define kTmpLeftSurr        2
#define kTmpRightSurr       3
#define kTmpCenter          4
#define kTmpCenterSurr      5

#define WA_RVB_TMPBUF_SIZE  256     /* temp buffer size in samples */

/*
 * Room simulation effect is essentially a surround->surround reverb.
 * It is based on an internal stereo->surround reverb. Input signals
 * are downmixed to stereo, run through the stereo->surround reverb,
 * and mixed with the input.
 */
typedef struct {
    int size;  /* for activate/deactivate */
    void *addr; /* for activate/deactivate */

    WaRvbStaticParam param;     /* static parameters */
    WaRvbDynParam dynParam;     /* dynamic parameters */
    float32 fxMixCoeff;         /* effects mix coefficient */
    float32 fxMixCoeffTarget;   /* target value of interpolation */
    float32 fxMixCoeffIncr;     /* coefficient increment */
    int32 tickCount;        /* counter for periodic ticks */
    int32 tickLen;          /* length of ticks */
    WaSurrRvb *fx;          /* 2->4 channel reverb effect */
    StereoDownMixer *sdm;   /* downmixer */
    DownSamp2 *ds;          /* downsampler */
    UpSamp2 *us;            /* upsampler */
    WaSampType *tmpRevIn[WA_RVB_NUM_IN_TMP];    /* temp buffers for mixing reverb input */
    WaSampType *tmpRevOut[WA_RVB_NUM_OUT_TMP];  /* temp buffers for reverb output */
    const WaRvbPreset  *presets;

//Declarations to support Pre Delay
// add 2 delay lines for RVB Object PreDelay
#define MAX_CDELAY 2048
    WaDelay *LDelay;
    WaDelay *RDelay;

} WaRvb;


extern const WaRvbStaticParam gWaRvbStaticParam;
extern const WaRvbDynParam gWaRvbDynParam;

#ifdef  __cplusplus
}
#endif

#endif
