
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Header for stereo->surround reverb algorithm.
//


#ifndef _WA_SURRRVB_H
#define _WA_SURRRVB_H

#include "LFOC.h"
#include "TableCXY.h"
#include "WaSampType.h"
#include "WaTypes.h"
#include "fil_datatype.h"

#ifdef  __cplusplus
extern "C" {
#endif

/*
 * Type of reverb delay samples.
 */
typedef int16 WsrSample;
//typedef float WsrSample;

#define MIN_RT  0.5     // w.r.t. scale == 1.0
#define MAX_RT  30.0

#ifndef PI
#define PI  3.14159265
#endif

#define FX_CONFIG_FS    22050   // default configuration

#define kRefSampRate    44100   // all delay constants w.r.t. this

#define WA_SURRRVB_LFTRIM       1   // true for low frequency trim on input

#if 0  //This control has been transfered to LINKINITPARAMS

#define WA_SURRRVB_DUAL_DELAYS  0   // true for 8K + 4K delay lines, else single 16K delay

#if WA_SURRRVB_DUAL_DELAYS      // Extra Delays not available in 12k build
#define PREDELAY   0
#define SURDELAY   0
#else
#define PREDELAY   1
#define SURDELAY   1
#endif // Extra Delay Enable

#endif

#if FX_CONFIG_FS == 22050       // delay length for 22050 Hz

#define kDelayLenB  4096        // remaining parts of right chl

#if 0   //Now this is Member of WaSurrRvb Struct Initialised in Reset Funtion

#if WA_SURRRVB_DUAL_DELAYS     
#define kDelayLen   8192        // parts of right chl and left chl
#else
#define kDelayLen   16384       // single delay line
#endif //WA_SURRRVB_DUAL_DELAYS
#endif

#else
#error Unknown value of FX_CONFIG_FS
#endif

#define SCALE_12K   0.9     // uses just less than 12K samples at 24 kHz
//
// Defines to turn on and off various components of the algorithm
// for debugging/timing.
//
#define INPUT       1   // input, filters, and feedback
#define ALLPASS     1   // allpass filters
#define OUTPUT      1   // output taps
#define SURR_OUT    1   // true for quad output, else stereo out
#define VARTAP      1   // true if LFO + variable delay

//
// LFO/vartap/tick configuration.
//
//  fs (Hz) detune (cts)    fL (Hz) excurse Ltick   fT (Hz)
//  44100   3.4             0.25    112     128     344.5
//  22050   3.4             0.25    112/2   128     172.3
//  11025   6.8             0.5     112/4   64      172.3
//
// See comments for RoomInit(). Also see notes, 10/9/98, 12/12/97.
//
#define kTickLen    128
#define kLFORate    0.25
#define kExcurse    112         // at 44.1 kHz, 3.4 cents max detune

//
// Table of RT (-60 dB at 500 Hz) to feedback gain coeffs, for scale = 1.0.
//
static const TableCXYEntry roomGainTab[] = {
    { 1.08, 0.0 },  // by Schroeder integrals
    { 1.29, 0.1 },
    { 1.67, 0.2 },
    { 2.19, 0.3 },
    { 2.48, 0.4 },
    { 3.13, 0.5 },
    { 4.29, 0.6 },
    { 5.91, 0.7 },
    {  8, 0.8   },     // by ear
    { 15, 0.897 },
    { 18, 0.92  },
    { 24, 0.944 },
    { 30, 0.96  }
};
#define N_ROOM_GAIN_TAB NELEMS(roomGainTab)

#define kMaxScale   SCALE_12K       // max scale, depends on memory space
#define kMinScale   0.25            // min scale, arbitrary

/*
 * Channel indices for inputs and outputs. The reverb
 * itself is stereo -> quad.
 */
#define kWsrLeft            0
#define kWsrRight           1
#define kWsrLeftSurr        2
#define kWsrRightSurr       3

#define kWsrNumInChan   2
#if SURR_OUT
#define kWsrNumOutChan  4
#else
#define kWsrNumOutChan  2
#endif

typedef struct {

    int32 Rnode1;       // right chl nodes
    int32 Rnode2;
    int32 Rnode5;
    int32 Rnode6;
    int32 Rnode7;
    int32 Rnode8;
    int32 Rnode9;

    int32 Lnode1;       // left chl nodes
    int32 Lnode2;
    int32 Lnode5;
    int32 Lnode6;
    int32 Lnode7;

    int32 Lnode7a;      // vartap
    int32 Lnode7b;

    int32 Lnode8;
    int32 Lnode9;

    // 
    // Output tap addresses. Most output taps are taken from
    // above nodes for efficiency.
    //
    int32 Rtap1;        // right chl output taps
    int32 Ltap1;        // left chl output taps

    int32 Bnode1;
    int32 Bnode2;

} WaSurrRvbNodes;

typedef struct {

    float32 sampRate;   // as Scaled for Decimation.    
    int32 tickCount;    // samples remaining until next tick
    int32 tickLen;      // length of periodic tick in samples

    float32 fsScale;    // scaling atributed to sampling rate
    
    int32 kDelayLen;    // parts of right chl and left chl now is member of this Struct
                        // So that it can be modified runtime between
                        // Dual Delay and Single Delay Buffer 
    
    WsrSample *buf;     // pointer to delay line
    int32 offset;       // base offset into delay line

    WsrSample *bufB;    // pointer to delay line B
    int32 offsetB;      // base offset into delay line B

    int32 LBase;        // beginning of left chl delay
//  int32 reverbLen;    // total length of L+R delay at max scale, doesn't include predelay
    TableCXY gainTab;   // maps RT to feedback gain
#if WA_SURRRVB_LFTRIM
    //
    // biquad for first order low shelf on input - this could be 
    // implemented using first order filter
    //
    float32 *coeff[2];
    float32 *w[2];
    PAF_FilParam  *pParams;
#endif
//
    // Coefficients.
    //
    float32 dampingGainCoeff;
    float32 dampingGainCoeffTarget;
    float32 dampingGainCoeffIncr;
    float32 feedbackGainCoeff;
    float32 feedbackGainCoeffTarget;
    float32 feedbackGainCoeffIncr;

#if VARTAP
    LFOC lfoSin;                // vartap LFO
    float32 vartapSinGain;
    float32 vartapSinGain1m;
    int32 vartapSinAddr;
    int32 vartapSinAddr1;
    int32 excurse;          // vartap excursion
#endif

    float32 zR; // state for R chl damping filter
    float32 zL; // state for L chl damping filter

//Declarations to support Surround Delay
    int32 maxChlPDLen;  // maximum length of a single channel Surdelay
    int32 LpdStart;     // stereo Surdelay
    int32 LpdEnd;
    int32 RpdStart;
    int32 RpdEnd;

    //
    // Delay line tap locations, called "nodes"; the
    // output taps are called "taps". These are needed in RAM
    // so that the size of the reverb can be changed dynamically.
    //
    int32 Rnode1;       // right chl nodes
    int32 Rnode2;
    int32 Rnode5;
    int32 Rnode6;
    int32 Rnode7;
    int32 Rnode8;
    int32 Rnode9;

    int32 Lnode1;       // left chl nodes
    int32 Lnode2;
    int32 Lnode5;
    int32 Lnode6;
    int32 Lnode7;
#if VARTAP
    int32 Lnode7a;      // vartap
    int32 Lnode7b;
#endif
    int32 Lnode8;
    int32 Lnode9;

    // 
    // Output tap addresses. Most output taps are taken from
    // above nodes for efficiency.
    //
    int32 Rtap1;        // right chl output taps
    int32 Ltap1;        // left chl output taps

    int32 Bnode2;
    
    int32 Rdelay2;            //Made an Integer Was #define in Original Code 
    
    WaSurrRvbNodes *fix, *var;//Made fix & var of all Nodes as Interger's 
                              // these were #define's in Original Code This is for
                              // Compatability for 3 Cases of Delay's supported by RVB
                              //--RV 
    //
    // Parameters.
    //
    float32 roomSize;       // size param: 0 to 1
    float32 scale;          // actual size scaling factor applied to tap addresses
    float32 dampingFreq;    // damping freq in Hz.
#if WA_SURRRVB_LFTRIM
    float32 lfTrimFreq;     // low shelf frequency
    float32 lfTrimHeight;   // low shelf height
#endif
    float32 reverbTime;     // reverb time in sec
#if VARTAP
    float32 lfoRate;        // lfo rate in Hz
#endif
    float32 frontOutGain;       /* L, R channel output gain */
    float32 centerOutGain;      /* C, SW channel output gain */
    float32 surrOutGain;        /* LS, RS, LBS, RBS channel output gain */
} WaSurrRvb;


#ifdef  __cplusplus
}
#endif

#endif

