
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Copy input to output implementation
//
//
//

#include <std.h>
#include <bufcpy.h>

// -----------------------------------------------------------------------------
// Example data processing function.
//    Copies input to output

int bufcpy (
            int * restrict pIn,
            int * restrict pOut,
            int numChanRx,
            int numChanTx,
            int wrdSizeRx,
            int wrdSizeTx,
            int rateX,
            int blockSize)
{
    Int i, j;


  if (rateX == 1) {    
    if ((wrdSizeRx == 4) && (wrdSizeTx == 4)) {
        if ((numChanRx == 8) && (numChanTx == 8)) {
            for (i=0; i < blockSize; i++)
                for (j=0; j < 8; j++)
                    pOut[i*8+j] = pIn[i*8+j];
        }
        else if ((numChanRx == 4) && (numChanTx == 8)) {
        //         Input --> output
        // chan 0 (left) --> chans 0 and 1 (left)
        // chan 1 (left) --> chans 2 and 3 (left)
        // chan 0 (rght) --> chans 0 and 1 (rght)
        // chan 1 (lght) --> chans 2 and 3 (rght)
        for(i = 0; i < blockSize; i++) {
          pOut[i*8+0] = pIn[i*4+0];
          pOut[i*8+1] = pIn[i*4+0];
          pOut[i*8+2] = pIn[i*4+1];
          pOut[i*8+3] = pIn[i*4+1];
          pOut[i*8+4] = pIn[i*4+2];
          pOut[i*8+5] = pIn[i*4+2];
          pOut[i*8+6] = pIn[i*4+3];
          pOut[i*8+7] = pIn[i*4+3];
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 8)) {
        //         Input --> output
        // chan 0 (left) --> chans 0, 1, 2, and 3 (left)
        // chan 0 (rght) --> chans 0, 1, 2, and 3 (rght)
        for(i = 0; i < blockSize; i++) {
          pOut[i*8+0] = pIn[i*2+0];
          pOut[i*8+1] = pIn[i*2+0];
          pOut[i*8+2] = pIn[i*2+0];
          pOut[i*8+3] = pIn[i*2+0];
          pOut[i*8+4] = pIn[i*2+1];
          pOut[i*8+5] = pIn[i*2+1];
          pOut[i*8+6] = pIn[i*2+1];
          pOut[i*8+7] = pIn[i*2+1];
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 2)) {
        //         Input --> output
        // chan 0 (left) --> chan 0 (left)
        // chan 0 (rght) --> chan 0 (rght)
        for(i = 0; i < blockSize; i++) {
          pOut[i*2+0] = pIn[i*2+0];
          pOut[i*2+1] = pIn[i*2+1];
        }
      }
      else if ((numChanRx == 4) && (numChanTx == 2)) {
        //                              Input --> output
        // chan 0 (left)*.5 + chan 0 (rght)*.5 --> chan 0 (left)
        // chan 1 (left)*.5 + chan 1 (rght)*.5 --> chan 0 (rght)
        for(i = 0; i < blockSize; i++) {
          pOut[i*2+0] = pIn[i*4+0]/2 + pIn[i*4+2]/2;
          pOut[i*2+1] = pIn[i*4+1]/2 + pIn[i*4+3]/2;
        }
      }
      else
        return 1;
      return 0;
    } //((wrdSizeRx == 4) && (wrdSizeTx == 4)) 
    else if ((wrdSizeRx == 4) && (wrdSizeTx == 3)) {
      if ((numChanRx == 2) && (numChanTx == 2)) {
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*6+2] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*6+1] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*6+0] = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*6+5] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*6+4] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*6+3] = (pIn[i*2+1] & 0x0000FF00) >> 8;
        }
      }
      else if ((numChanRx == 8) && (numChanTx == 2)) {
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*6+2] = (pIn[i*8+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*6+1] = (pIn[i*8+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*6+0] = (pIn[i*8+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*6+5] = (pIn[i*8+4] & 0xFF000000) >> 24;
          ((char *)pOut)[i*6+4] = (pIn[i*8+4] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*6+3] = (pIn[i*8+4] & 0x0000FF00) >> 8;
        }
      }
	  else if ((numChanRx == 6) && (numChanTx == 2)) {
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*6+2] = (pIn[i*6+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*6+1] = (pIn[i*6+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*6+0] = (pIn[i*6+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*6+5] = (pIn[i*6+3] & 0xFF000000) >> 24;
          ((char *)pOut)[i*6+4] = (pIn[i*6+3] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*6+3] = (pIn[i*6+3] & 0x0000FF00) >> 8;
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 8)) {
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*24+2]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+1]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+0]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*24+5]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+4]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+3]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*24+8]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+7]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+6]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*24+11] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+10] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+9]  = (pIn[i*2+0] & 0x0000FF00) >> 8;

          ((char *)pOut)[i*24+14] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+13] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+12] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*24+17] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+16] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+15] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*24+20] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+19] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+18] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*24+23] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*24+22] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*24+21] = (pIn[i*2+1] & 0x0000FF00) >> 8;
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 6)) {
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*18+2]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+1]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+0]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+5]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+4]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+3]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+8]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+7]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+6]  = (pIn[i*2+0] & 0x0000FF00) >> 8;

          ((char *)pOut)[i*18+11] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+10] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+9]  = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+14] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+13] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+12] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+17] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+16] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+15] = (pIn[i*2+1] & 0x0000FF00) >> 8;
        }
      }
      else if ((numChanRx == 6) && (numChanTx == 6)) {
          for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*18+2]  = (pIn[i*6+0] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+1]  = (pIn[i*6+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+0]  = (pIn[i*6+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+5]  = (pIn[i*6+1] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+4]  = (pIn[i*6+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+3]  = (pIn[i*6+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+8]  = (pIn[i*6+2] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+7]  = (pIn[i*6+2] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+6]  = (pIn[i*6+2] & 0x0000FF00) >> 8;

          ((char *)pOut)[i*18+11] = (pIn[i*6+3] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+10] = (pIn[i*6+3] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+9]  = (pIn[i*6+3] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+14] = (pIn[i*6+4] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+13] = (pIn[i*6+4] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+12] = (pIn[i*6+4] & 0x0000FF00) >> 8;
          ((char *)pOut)[i*18+17] = (pIn[i*6+5] & 0xFF000000) >> 24;
          ((char *)pOut)[i*18+16] = (pIn[i*6+5] & 0x00FF0000) >> 16;
          ((char *)pOut)[i*18+15] = (pIn[i*6+5] & 0x0000FF00) >> 8;          }
      }
      else if ((numChanRx == 8) && (numChanTx == 8)) {
          for(i = 0; i < blockSize; i++) {
              for (j=0; j < 2; j++) {
                  int in0 = ((int *)pIn)[i*8+4*j+0];
                  int in1 = ((int *)pIn)[i*8+4*j+1];
                  int in2 = ((int *)pIn)[i*8+4*j+2];
                  int in3 = ((int *)pIn)[i*8+4*j+3];

                  int lsb0 = ((in0 & 0x0000FF00) >> 8);
                  int msb0 = ((in0 & 0x00FF0000) >> 8);
                  int hsb0 = ((in0 & 0xFF000000) >> 8);
                  int lsb1 = ((in1 & 0x0000FF00) << 16);

                  int msb1 = ((in1 & 0x00FF0000) >> 16);
                  int hsb1 = ((in1 & 0xFF000000) >> 16);
                  int lsb2 = ((in2 & 0x0000FF00) << 8);
                  int msb2 = ((in2 & 0x00FF0000) << 8);
                  int hsb2 = ((in2 & 0xFF000000) >> 24);
                  int lsb3 = ((in3 & 0x0000FF00));
                  int msb3 = ((in3 & 0x00FF0000));
                  int hsb3 = ((in3 & 0xFF000000));
                  
                  int out0 = lsb0 | msb0 | hsb0 | lsb1;
                  int out1 = msb1 | hsb1 | lsb2 | msb2;
                  int out2 = hsb2 | lsb3 | msb3 | hsb3;

                  ((int *)pOut)[i*6+3*j+0] = out0;
                  ((int *)pOut)[i*6+3*j+1] = out1;
                  ((int *)pOut)[i*6+3*j+2] = out2;
              }
          }
      }
    } //((wrdSizeRx == 4) && (wrdSizeTx == 3))
    else if ((wrdSizeRx == 2) && (wrdSizeTx == 4)) {
        if ((numChanRx == 2) && (numChanTx == 2)) {
            //         Input --> output
            // chan 0 (left) --> chan 0 (left)
            // chan 0 (rght) --> chan 0 (rght)
            for(i = 0; i < blockSize; i++) {
                pOut[i*2+0] = ((short *)pIn)[i*2+0] << 16;
                pOut[i*2+1] = ((short *)pIn)[i*2+1] << 16;
            }
        }
        else if ((numChanRx == 2) && (numChanTx == 8)) {
            for(i = 0; i < blockSize; i++) {
                pOut[i*8+0] = ((short *)pIn)[i*2+0] << 16;
                pOut[i*8+1] = ((short *)pIn)[i*2+0] << 16;
                pOut[i*8+2] = ((short *)pIn)[i*2+0] << 16;
                pOut[i*8+3] = ((short *)pIn)[i*2+0] << 16;
                pOut[i*8+4] = ((short *)pIn)[i*2+1] << 16;
                pOut[i*8+5] = ((short *)pIn)[i*2+1] << 16;
                pOut[i*8+6] = ((short *)pIn)[i*2+1] << 16;
                pOut[i*8+7] = ((short *)pIn)[i*2+1] << 16;
            }
        }
    } //((wrdSizeRx == 2) && (wrdSizeTx == 4))
    else if ((wrdSizeRx == 2) && (wrdSizeTx == 3)) {
      if ((numChanRx == 2) && (numChanTx == 2)) {
        //         Input --> output
        // chan 0 (left) --> chan 0 (left)
        // chan 0 (rght) --> chan 0 (rght)
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[i*6+2] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[i*6+1] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[i*6+0] = 0;
          ((char *)pOut)[i*6+5] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[i*6+4] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[i*6+3] = 0;
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 8)) {
        //         Input --> output
        // chan 0 (left) --> chan 0 (left)
        // chan 0 (rght) --> chan 0 (rght)
          for(i = 0; i < blockSize; i++) {
              short inpLeft = ((short *)pIn)[i*2+0];
              short inpRght = ((short *)pIn)[i*2+1];
              unsigned char leftMSB = (inpLeft & 0xFF00) >> 8;
              unsigned char leftLSB = (inpLeft & 0x00FF);
              unsigned char rghtMSB = (inpRght & 0xFF00) >> 8;
              unsigned char rghtLSB = (inpRght & 0x00FF);
              int word0 = (leftMSB << 16) | (leftLSB << 8);
              int word1 = (leftLSB << 24) | (leftMSB << 8) | leftLSB;
              int word2 = (leftMSB << 24) | (leftLSB << 16) | leftMSB;
              int word3 = (rghtMSB << 16) | (rghtLSB << 8);
              int word4 = (rghtLSB << 24) | (rghtMSB << 8) | rghtLSB;
              int word5 = (rghtMSB << 24) | (rghtLSB << 16) | rghtMSB;

              ((int *)pOut)[i*6+0] = word0;
              ((int *)pOut)[i*6+1] = word1;
              ((int *)pOut)[i*6+2] = word2;
              ((int *)pOut)[i*6+3] = word3;
              ((int *)pOut)[i*6+4] = word4;
              ((int *)pOut)[i*6+5] = word5;
          }
      }
    } //((wrdSizeRx == 2) && (wrdSizeTx == 3))
    else if ((wrdSizeRx == 2) && (wrdSizeTx == 2)) {
      if ((numChanRx == 2) && (numChanTx == 2)) {
        //         Input --> output
        // chan 0 (left) --> chan 0 (left)
        // chan 0 (rght) --> chan 0 (rght)
		memcpy(pOut,pIn,numChanTx*blockSize*wrdSizeRx);
      }
    } //((wrdSizeRx == 2) && (wrdSizeTx == 2))
  }
  // if double rate output then output buffer is twice as large
  // perform simple, 0th order, interpolation
  else if (rateX == 2) {
    if ((wrdSizeRx == 4) && (wrdSizeTx == 3)) {
      if ((numChanRx == 2) && (numChanTx == 2)) {
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[    j*6+2] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*6+1] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*6+0] = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*6+2] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*6+1] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*6+0] = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[    j*6+5] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*6+4] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*6+3] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*6+5] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*6+4] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*6+3] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          j +=2;
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 8)) {
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[    j*24+2]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+1]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+0]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+2]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+1]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+0]  = (pIn[i*2+0] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+5]  = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+4]  = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+3]  = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+5]  = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+4]  = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+3]  = (pIn[i*2+1] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+8]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+7]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+6]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+8]  = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+7]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+6]  = (pIn[i*2+0] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+11] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+10] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+9]  = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+11] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+10] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+9]  = (pIn[i*2+1] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+14] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+13] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+12] = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+14] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+13] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+12] = (pIn[i*2+0] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+17] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+16] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+15] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+17] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+16] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+15] = (pIn[i*2+1] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+20] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+19] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+18] = (pIn[i*2+0] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+20] = (pIn[i*2+0] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+19] = (pIn[i*2+0] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+18] = (pIn[i*2+0] & 0x0000FF00) >> 8;

          ((char *)pOut)[    j*24+23] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[    j*24+22] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[    j*24+21] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          ((char *)pOut)[(j+1)*24+23] = (pIn[i*2+1] & 0xFF000000) >> 24;
          ((char *)pOut)[(j+1)*24+22] = (pIn[i*2+1] & 0x00FF0000) >> 16;
          ((char *)pOut)[(j+1)*24+21] = (pIn[i*2+1] & 0x0000FF00) >> 8;
          j+=2;
        }
      }
    } //((wrdSizeRx == 4) && (wrdSizeTx == 3)) {
    else if ((wrdSizeRx == 2) && (wrdSizeTx == 3)) {
      if ((numChanRx == 2) && (numChanTx == 2)) {
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[    j*6+2] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[    j*6+1] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[    j*6+0] = 0;
          ((char *)pOut)[(j+1)*6+2] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*6+1] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[(j+1)*6+0] = 0;
          ((char *)pOut)[    j*6+5] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[    j*6+4] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[    j*6+3] = 0;
          ((char *)pOut)[(j+1)*6+5] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*6+4] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[(j+1)*6+3] = 0;
          j +=2;
        }
      } 
      else if ((numChanRx == 2) && (numChanTx == 8)) {
        int j= 0;
        for(i = 0; i < blockSize; i++) {
          ((char *)pOut)[    j*24+2] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+1] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[    j*24+0] = 0;
          ((char *)pOut)[(j+1)*24+2] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+1] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[(j+1)*24+0] = 0;

          ((char *)pOut)[    j*24+5] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+4] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[    j*24+3] = 0;
          ((char *)pOut)[(j+1)*24+5] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+4] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[(j+1)*24+3] = 0;

          ((char *)pOut)[    j*24+8] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+7] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[    j*24+6] = 0;
          ((char *)pOut)[(j+1)*24+8] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+7] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[(j+1)*24+6] = 0;

          ((char *)pOut)[    j*24+11] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+10] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[    j*24+9] = 0;
          ((char *)pOut)[(j+1)*24+11] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+10] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[(j+1)*24+9] = 0;

          ((char *)pOut)[    j*24+14] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+13] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[    j*24+12] = 0;
          ((char *)pOut)[(j+1)*24+14] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+13] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[(j+1)*24+12] = 0;

          ((char *)pOut)[    j*24+17] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+16] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[    j*24+15] = 0;
          ((char *)pOut)[(j+1)*24+17] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+16] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[(j+1)*24+15] = 0;

          ((char *)pOut)[    j*24+20] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+19] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[    j*24+18] = 0;
          ((char *)pOut)[(j+1)*24+20] = (((short *)pIn)[i*2+0] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+19] = (((short *)pIn)[i*2+0] & 0x00FF);
          ((char *)pOut)[(j+1)*24+18] = 0;

          ((char *)pOut)[    j*24+23] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[    j*24+22] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[    j*24+21] = 0;
          ((char *)pOut)[(j+1)*24+23] = (((short *)pIn)[i*2+1] & 0xFF00) >> 8;
          ((char *)pOut)[(j+1)*24+22] = (((short *)pIn)[i*2+1] & 0x00FF);
          ((char *)pOut)[(j+1)*24+21] = 0;
          j+=2; 
        }
      }
    } //((wrdSizeRx == 2) && (wrdSizeTx == 3)) {
    else if ((wrdSizeRx == 4) && (wrdSizeTx == 4)) {
      if ((numChanRx == 4) && (numChanTx == 8)) {
        //         Input --> output
        // chan 0 (left) --> chans 0 and 1 (left)
        // chan 1 (left) --> chans 2 and 3 (left)
        // chan 0 (rght) --> chans 0 and 1 (rght)
        // chan 1 (lght) --> chans 2 and 3 (rght)
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          pOut[    j*8+0] = pIn[i*4+0];
          pOut[(j+1)*8+0] = pIn[i*4+0];
          pOut[    j*8+1] = pIn[i*4+0];
          pOut[(j+1)*8+1] = pIn[i*4+0];
          pOut[    j*8+2] = pIn[i*4+1];
          pOut[(j+1)*8+2] = pIn[i*4+1];
          pOut[    j*8+3] = pIn[i*4+1];
          pOut[(j+1)*8+3] = pIn[i*4+1];
          pOut[    j*8+4] = pIn[i*4+2];
          pOut[(j+1)*8+4] = pIn[i*4+2];
          pOut[    j*8+5] = pIn[i*4+2];
          pOut[(j+1)*8+5] = pIn[i*4+2];
          pOut[    j*8+6] = pIn[i*4+3];
          pOut[(j+1)*8+6] = pIn[i*4+3];
          pOut[    j*8+7] = pIn[i*4+3];
          pOut[(j+1)*8+7] = pIn[i*4+3];
          j+=2;
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 8)) {
        //         Input --> output
        // chan 0 (left) --> chans 0, 1, 2, and 3 (left)
        // chan 0 (rght) --> chans 0, 1, 2, and 3 (rght)
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          pOut[    j*8+0] = pIn[i*2+0];
          pOut[(j+1)*8+0] = pIn[i*2+0];
          pOut[    j*8+1] = pIn[i*2+0];
          pOut[(j+1)*8+1] = pIn[i*2+0];
          pOut[    j*8+2] = pIn[i*2+0];
          pOut[(j+1)*8+2] = pIn[i*2+0];
          pOut[    j*8+3] = pIn[i*2+0];
          pOut[(j+1)*8+3] = pIn[i*2+0];
          pOut[    j*8+4] = pIn[i*2+1];
          pOut[(j+1)*8+4] = pIn[i*2+1];
          pOut[    j*8+5] = pIn[i*2+1];
          pOut[(j+1)*8+5] = pIn[i*2+1];
          pOut[    j*8+6] = pIn[i*2+1];
          pOut[(j+1)*8+6] = pIn[i*2+1];
          pOut[    j*8+7] = pIn[i*2+1];
          pOut[(j+1)*8+7] = pIn[i*2+1];
          j+=2;
        }
      }
      else if ((numChanRx == 2) && (numChanTx == 2)) {
        //         Input --> output
        // chan 0 (left) --> chan 0 (left)
        // chan 0 (rght) --> chan 0 (rght)
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          pOut[    j*2+0] = pIn[i*2+0];
          pOut[(j+1)*2+0] = pIn[i*2+0];
          pOut[    j*2+1] = pIn[i*2+1];
          pOut[(j+1)*2+1] = pIn[i*2+1];
          j+=2;
        }
      }
      else if ((numChanRx == 4) && (numChanTx == 2)) {
        //                              Input --> output
        // chan 0 (left)*.5 + chan 0 (rght)*.5 --> chan 0 (left)
        // chan 1 (left)*.5 + chan 1 (rght)*.5 --> chan 0 (rght)
        int j = 0;
        for(i = 0; i < blockSize; i++) {
          pOut[    j*2+0] = pIn[i*4+0]/2 + pIn[i*4+2]/2;
          pOut[(j+1)*2+0] = pOut[j*2+0];
          pOut[    j*2+1] = pIn[i*4+1]/2 + pIn[i*4+3]/2;
          pOut[(j+1)*2+1] = pOut[j*2+1];
          j+=2;
        }
      }
      else 
        return 1;
    } ////((wrdSizeRx == 4) && (wrdSizeTx == 4))
  }
  else
    return 1;
    
  return 0;
} //bufcpy

// -----------------------------------------------------------------------------

int bufcpy2 (
            int * restrict pIn,
            int * restrict pOut,
            int numChanRx,
            int numChanTx,
            int wrdSizeRx,
            int wrdSizeTx,
            int rateX,
            int blockSize,
            int rxMask, 
            int txMask)
{
    int i;

    if ((wrdSizeRx == 4) && (wrdSizeTx == 3)) {
        if ((numChanRx == 2) && (numChanTx == 8)) {
            for(i = 0; i < blockSize; i++) {
                if (txMask & (1 << 0)) {
                    ((char *)pOut)[i*24+2]  = (pIn[i*2+0] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+1]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+0]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 1)) {
                    ((char *)pOut)[i*24+5]  = (pIn[i*2+0] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+4]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+3]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 2)) {
                    ((char *)pOut)[i*24+8]  = (pIn[i*2+0] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+7]  = (pIn[i*2+0] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+6]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 3)) {
                    ((char *)pOut)[i*24+11] = (pIn[i*2+0] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+10] = (pIn[i*2+0] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+9]  = (pIn[i*2+0] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 4)) {
                    ((char *)pOut)[i*24+14] = (pIn[i*2+1] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+13] = (pIn[i*2+1] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+12] = (pIn[i*2+1] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 5)) {
                    ((char *)pOut)[i*24+17] = (pIn[i*2+1] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+16] = (pIn[i*2+1] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+15] = (pIn[i*2+1] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 6)) {
                    ((char *)pOut)[i*24+20] = (pIn[i*2+1] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+19] = (pIn[i*2+1] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+18] = (pIn[i*2+1] & 0x0000FF00) >> 8;
                }
                if (txMask & (1 << 7)) {
                    ((char *)pOut)[i*24+23] = (pIn[i*2+1] & 0xFF000000) >> 24;
                    ((char *)pOut)[i*24+22] = (pIn[i*2+1] & 0x00FF0000) >> 16;
                    ((char *)pOut)[i*24+21] = (pIn[i*2+1] & 0x0000FF00) >> 8;
                }
            }
        }
        else
            return 1;
    } //((wrdSizeRx == 4) && (wrdSizeTx == 3))
    else
        return 1;

    return 0;
} //bufcpy2
// -----------------------------------------------------------------------------
