
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Echo program for audio loopback on Primus FPGA
//
//
//

#include <std.h>
#include <log.h>
#include <tsk.h>
#include <dap_e17.h>
#include <pafsio.h>
#include <paftyp_a.h>

#include <bufcpy.h>

#include <dmax_struct.h>

extern LOG_Obj trace;

static void testINTC (void);
static void testINTC_isr (void);
static void testLoopback (void);

extern Int IRAM;

char *sampleRates[PAF_SAMPLERATE_192000HZ+1]=
{
    "Sample rate Unknown",
    "None",
    "32   kHz",
    "44.1 kHz",
    "48   kHz",
    "88.2 kHz",
    "96   kHz",
    "192  kHz"
};

char *lockStates[2] =
{
    "Unlocked",
    "Locked"
};

char *audioStates[3] =
{
    "Audio Unknown",
    "Audio",
    "NonAudio"
};

char *emphasisStates[3] =
{
    "Emphasis Unknown",
    "Emphasis No",
    "Emphasis Yes"
};

#define INPUT   DAP_E17_RX_DIR//DAP_S2_RX_ADC1 // //DAP_E5_ADC_48000HZ // //DAP_E5_DIR // //
#define OUTPUT DAP_E17_TX_DAC // //DAP_S2_TX_DAC1

int  wrdSizeRx = 2;

static void printStatus (PAF_SIO_InputStatus *pStatus);

#define SYS_BASE            0x01C14000

#define PINMUX0				*(volatile unsigned int*)(SYS_BASE + 0x120)	//PINMUX0
#define PINMUX1				*(volatile unsigned int*)(SYS_BASE + 0x124)	//PINMUX1
#define PINMUX2				*(volatile unsigned int*)(SYS_BASE + 0x128)	//PINMUX2
#define PINMUX3				*(volatile unsigned int*)(SYS_BASE + 0x12C)	//PINMUX3
#define PINMUX4				*(volatile unsigned int*)(SYS_BASE + 0x130)	//PINMUX4
#define PINMUX5				*(volatile unsigned int*)(SYS_BASE + 0x134)	//PINMUX5
#define PINMUX6				*(volatile unsigned int*)(SYS_BASE + 0x138)	//PINMUX6
#define PINMUX7				*(volatile unsigned int*)(SYS_BASE + 0x13C)	//PINMUX7
#define PINMUX8				*(volatile unsigned int*)(SYS_BASE + 0x140)	//PINMUX8
#define PINMUX9				*(volatile unsigned int*)(SYS_BASE + 0x144)	//PINMUX9
#define PINMUX10			*(volatile unsigned int*)(SYS_BASE + 0x148)	//PINMUX10
#define PINMUX11			*(volatile unsigned int*)(SYS_BASE + 0x14C)	//PINMUX11
#define PINMUX12			*(volatile unsigned int*)(SYS_BASE + 0x150)	//PINMUX12
#define PINMUX13			*(volatile unsigned int*)(SYS_BASE + 0x154)	//PINMUX13
#define PINMUX14			*(volatile unsigned int*)(SYS_BASE + 0x158)	//PINMUX14
#define PINMUX15			*(volatile unsigned int*)(SYS_BASE + 0x15C)	//PINMUX15
#define PINMUX16			*(volatile unsigned int*)(SYS_BASE + 0x160)	//PINMUX16
#define PINMUX17			*(volatile unsigned int*)(SYS_BASE + 0x164)	//PINMUX17
#define PINMUX18			*(volatile unsigned int*)(SYS_BASE + 0x168)	//PINMUX18
#define PINMUX19			*(volatile unsigned int*)(SYS_BASE + 0x16C)	//PINMUX19

#define KICK0R              *(unsigned int*)(SYS_BASE + 0x038)
#define KICK1R              *(unsigned int*)(SYS_BASE + 0x03c)

/*PSC Module Related Registers*/
#define PSC0_BASE       0x01C10000
#define PSC1_BASE       0x01E27000
#define PSC0_MDCTL      (PSC0_BASE+0xA00)
#define PSC0_MDSTAT     (PSC0_BASE+0x800)
#define PSC0_PTCMD      *(unsigned int*) (PSC0_BASE + 0x120)
#define PSC0_PTSTAT     *(unsigned int*) (PSC0_BASE + 0x128)

#define PSC1_MDCTL      (PSC1_BASE+0xA00)
#define PSC1_MDSTAT     (PSC1_BASE+0x800)
#define PSC1_PTCMD      *(unsigned int*) (PSC1_BASE + 0x120)
#define PSC1_PTSTAT     *(unsigned int*) (PSC1_BASE + 0x128)

/*Enable Function for PSC0*/
void PSC0_lPSC_enable(unsigned int PD, unsigned int LPSC_num)
{

    *(unsigned int*) (PSC0_MDCTL+4*LPSC_num) = (*(unsigned int*) (PSC0_MDCTL+4*LPSC_num) & 0xFFFFFFE0) | 0x0003;
    PSC0_PTCMD = 0x1<<PD;
    while( (PSC0_PTSTAT & (0x1<<PD) ) !=0) ; /*Wait for power state transition to finish*/
    while( (*(unsigned int*)(PSC0_MDSTAT+4 * LPSC_num) & 0x1F) !=0x3);
}

/*Enable Function for PSC1*/
void PSC1_lPSC_enable(unsigned int PD, unsigned int LPSC_num)
{

    *(unsigned int*) (PSC1_MDCTL+4*LPSC_num) = (*(unsigned int*) (PSC1_MDCTL+4*LPSC_num) & 0xFFFFFFE0) | 0x0003;
    PSC1_PTCMD = 0x1<<PD;
    while( (PSC1_PTSTAT & (0x1<<PD) ) !=0) ; /*Wait for power state transition to finish*/
    while( (*(unsigned int*)(PSC1_MDSTAT+4 * LPSC_num) & 0x1F) !=0x3);
}
// .............................................................................
// -----------------------------------------------------------------------------
#if defined(__TMS470__)
#define TARGET_ISA 'A'    // CPU = ARM926EJ
#elif defined(_TMS320C6X)
#define TARGET_ISA 'D'    // CPU = Gem DSP
#else
#define TARGET_ISA '?'    // CPU = ?, should never happen
#endif
void main ()
{

    KICK0R = 0x83e70b13;  // Kick0 register + data (unlock)
    KICK1R = 0x95a4f1e0;  // Kick1 register + data (unlock)

    PINMUX0  = 0x11111188;  // EMIFB, Check EMU0/RTCK
    PINMUX1  = 0x11111111;  // EMIFB
    PINMUX2  = 0x11111111;  // EMIFB
    PINMUX3  = 0x11111111;  // EMIFB
    PINMUX4  = 0x11111111;  // EMIFB
    PINMUX5  = 0x11111111;  // EMIFB
    PINMUX6  = 0x11111111;  // EMIFB
    PINMUX7  = 0x11111111;  // EMIFB, SPI0
    PINMUX8  = 0x21122111;  // UART2, McASP1, I2C0, I2C1
    PINMUX9  = 0x11011112;  // RMII CLK, McASP0, USB_DRVVBUS, UART2
    PINMUX10 = 0x22222221;  // RMII/ McASP0
    PINMUX11 = 0x11142222;  // McASP1, UART1, McASP0, MDIO (last 2 digits 0x22 for MDIO instead of GPIO)
    PINMUX12 = 0x11111111;  // McASP0 / McASP1
    PINMUX13 = 0x22111111;  // SD / McASP1
    PINMUX14 = 0x88222222;  // SD / EMIFA
    PINMUX15 = 0x21888888;  // SD / EMIFA
    PINMUX16 = 0x11111112;  // SD / EMIFA
    PINMUX17 = 0x00100111;  // EMIFA
    PINMUX18 = 0x11111111;  // EMIFA
    PINMUX19 = 0x00000001;  // EMIFA

#if 1
    // PSC0
    PSC0_lPSC_enable(0, 0);
    PSC0_lPSC_enable(0, 1);
    PSC0_lPSC_enable(0, 2);
    PSC0_lPSC_enable(0, 3);  // EMIFA
    PSC0_lPSC_enable(0, 4);
    PSC0_lPSC_enable(0, 5);
    PSC0_lPSC_enable(0, 6);
    PSC0_lPSC_enable(0, 8);
    PSC0_lPSC_enable(0, 9);
    PSC0_lPSC_enable(0, 10);
    PSC0_lPSC_enable(0, 11);
    PSC0_lPSC_enable(0, 12);
    PSC0_lPSC_enable(0, 13);

    // PSC1
    PSC1_lPSC_enable(0, 1);
    PSC1_lPSC_enable(0, 2);
    PSC1_lPSC_enable(0, 3);
    PSC1_lPSC_enable(0, 4);
    PSC1_lPSC_enable(0, 5);
    PSC1_lPSC_enable(0, 6);  // EMIFB
    PSC1_lPSC_enable(0, 7);
    PSC1_lPSC_enable(0, 8);
    PSC1_lPSC_enable(0, 9);
    PSC1_lPSC_enable(0, 10);
    PSC1_lPSC_enable(0, 11);
    PSC1_lPSC_enable(0, 12);
    PSC1_lPSC_enable(0, 13);
    PSC1_lPSC_enable(0, 16);
    PSC1_lPSC_enable(0, 17);
    PSC1_lPSC_enable(0, 20);
    PSC1_lPSC_enable(0, 21);
    PSC1_lPSC_enable(0, 24);
    PSC1_lPSC_enable(0, 25);
    PSC1_lPSC_enable(0, 26);
    PSC1_lPSC_enable(0, 31);
#endif
    // initialize dMAX
    dMAX_init (dMAX_HANDLE);
} //main

// -----------------------------------------------------------------------------

void echoTask (void)
{
    testLoopback ();

} //echoTask

// ----------------------------------------------------------------------------
int fail_flag_rx=1;
int fail_flag_tx=1;
int frame;
unsigned int gecho=0;
unsigned int gecho1=0;
unsigned int gecho2=0;
unsigned int gecho3=0;
unsigned int gecho4=0;
unsigned int gecho5=0;
#define BLOCK_SIZE 256
#define NUM_FRAME 2
#define RATE_MULT_TX 1 // output rate multiplier (1=normal, 2=double rate output)
unsigned int ts2,ts1,tii2,tri1,tro1;
unsigned int TestSrc[BLOCK_SIZE*8*NUM_FRAME];
#include "c6x.h"
void testSrc_init()
{
	int j;
	for (j=0;j<BLOCK_SIZE*8*NUM_FRAME;j++)
		TestSrc[j]=(j<<24)+((j+1)<<16)+((j+2)<<8)+(j+3);
}
unsigned int gwhileFlag=0;
static void testLoopback (void)
{
    SIO_Attrs sioAttrs;
    SIO_Handle hRxSio, hTxSio;
    int i,errno;
    Ptr pTxBuf, pRxBuf;
    int start, diff;

	int nbytes;
    float ratex = (float) RATE_MULT_TX;

    int numChanRx, numChanTx;
    int bufSizeRx, bufSizeTx;
    int wrdSizeTx;
    Ptr txPtr[2], rxPtr[2];

    PAF_SIO_InputStatus lastStatus, curStatus;

    sioAttrs.nbufs = 2;              // double buffering
    sioAttrs.segid = IRAM;
    sioAttrs.align = 128;
    sioAttrs.flush = FALSE;
    sioAttrs.model = SIO_ISSUERECLAIM;
    sioAttrs.timeout = SYS_FOREVER;

    //RX Alloc and Config
#ifdef INPUT
    hRxSio = SIO_create ("/DAP", SIO_INPUT, 0, &sioAttrs);
    if (!hRxSio) {
        LOG_printf (&trace, "Failed to create hRxSio. Aborting");
        return;
    }
    errno = SIO_ctrl (hRxSio, PAF_SIO_CONTROL_OPEN, (Arg)&INPUT);
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to open hRxSio. Aborting", errno);
        return;
    }
    errno = SIO_ctrl (hRxSio, PAF_SIO_CONTROL_GET_NUMCHANNELS, (Arg) &numChanRx );
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to getNumChannels for hRxSio. Aborting", errno);
        return;
    }

    errno = SIO_ctrl (hRxSio, PAF_SIO_CONTROL_SET_WORDSIZE, (Arg) wrdSizeRx );
	if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to setWordSize for hRxSio. Aborting", errno);
        return;
    }
#endif

//--------------------
// Tx Alloc and Config

#ifdef OUTPUT
    hTxSio = SIO_create ("/DAP", SIO_OUTPUT, 0, &sioAttrs);
    if (!hTxSio) {
        LOG_printf (&trace, "Failed to create hTxSio. Aborting");
        TSK_sleep (10);
        return;
    }
    errno = SIO_ctrl (hTxSio, PAF_SIO_CONTROL_OPEN, (Arg)&OUTPUT);
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to open hTxSio. Aborting", errno);
        TSK_sleep (10);
        return;
    }
    errno = SIO_ctrl (hTxSio, PAF_SIO_CONTROL_GET_NUMCHANNELS, (Arg) &numChanTx);
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to getNumChannels for hTxSio. Aborting", errno);
        TSK_sleep (10);
        return;
    }
    errno = SIO_ctrl (hTxSio, PAF_SIO_CONTROL_GET_WORDSIZE, (Arg) &wrdSizeTx );
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to getWordSize for hTxSio. Aborting", errno);
        TSK_sleep (10);
        return;
    }
    errno = SIO_ctrl (hTxSio, PAF_SIO_CONTROL_UNMUTE, 0);
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to unmute. Aborting", errno);
        TSK_sleep (10);
        return;
    }
#endif

//-------------------
// IO Buffer Alloc

#ifdef INPUT
    bufSizeRx = BLOCK_SIZE*numChanRx*wrdSizeRx;
    LOG_printf (&trace, "bufSizeRx (bytes) = 0x%0x", bufSizeRx);
    LOG_printf (&trace, "Rx word size = 0x%0x", wrdSizeRx);
    LOG_printf (&trace, "# Input Channels = %d", numChanRx);
#endif
#ifdef OUTPUT
    bufSizeTx = BLOCK_SIZE*numChanTx*wrdSizeTx*RATE_MULT_TX;
    LOG_printf (&trace, "bufSizeTx (bytes) = 0x%0x", bufSizeTx);
    LOG_printf (&trace, "Tx word size = 0x%0x", wrdSizeTx);
    LOG_printf (&trace, "# Output Channels = %d", numChanTx);
    LOG_printf (&trace, "Tx Rate Multiplier = %d", RATE_MULT_TX);
#endif

    for (i=0; i < sioAttrs.nbufs; i++) {
#ifdef OUTPUT
        pTxBuf = (Ptr )MEM_calloc(IRAM, bufSizeTx, 128);
#ifdef BOARD_E4
        pTxBuf = (Ptr) (0xA0100000 | ((int) pTxBuf & 0x0000FFFF));
#endif
        if (!pTxBuf) {
            LOG_printf (&trace, "Failed to allocate Tx Buf # %d", i);
            TSK_sleep (10);
            return;
        }
        txPtr[i] = pTxBuf;

        LOG_printf (&trace," Tx buf #%d address = 0x%0x", i, pTxBuf);
#endif
#ifdef INPUT
        pRxBuf = (Ptr )MEM_calloc(IRAM, bufSizeRx, 128);
#ifdef BOARD_E4
        pRxBuf = (Ptr) (0xA0100000 | ((int) pRxBuf & 0x0000FFFF));
#endif
        if (!pRxBuf) {
            LOG_printf (&trace, "Failed to allocate Rx Buf # %d", i);
            TSK_sleep (10);
            return;
        }
        rxPtr[i] = pRxBuf;

        LOG_printf (&trace," Rx buf #%d address = 0x%0x", i, pRxBuf);
#endif
    }

//-----------------------
#ifdef INPUT
    lastStatus.lock = 0;
    lastStatus.nonaudio = 0;
    lastStatus.emphasis = 0;
    lastStatus.sampleRateMeasured = 0;
    curStatus = lastStatus;
#endif

    LOG_printf (&trace, "Looping back audio and tracking input status..." );
    start = 1;
    diff = 1;

    while (1)
	{
       gwhileFlag++;

        // if error condition, e.g. due CCS stop/start, then reset IO
        if (errno) {
#ifdef INPUT
            SIO_idle (hRxSio);
#endif
#ifdef OUTPUT
            SIO_idle (hTxSio);
#endif
            start = 1;
            errno = 0;
        }

#ifdef INPUT
        errno = SIO_ctrl (hRxSio, (Uns)PAF_SIO_CONTROL_GET_INPUT_STATUS, (Arg) &curStatus);
        if (errno) {
            LOG_printf( &trace, "Failed (0x%0x) to get input status. Aborting", errno);
            continue;
        }

        if (curStatus.lock != lastStatus.lock)
            diff = 1;

        if (curStatus.nonaudio != lastStatus.nonaudio)
            diff = 1;

        if (curStatus.emphasis != lastStatus.emphasis)
            diff = 1;

        if (curStatus.sampleRateMeasured != lastStatus.sampleRateMeasured)
            diff = 1;

        if (diff) {
            lastStatus = curStatus;
            printStatus (&curStatus);
            diff = 0;
        }

        if (!curStatus.lock) {
            if (diff)
                LOG_printf( &trace, "Waiting for Lock" );
            TSK_sleep(10);
#ifdef INPUT
            SIO_idle (hRxSio);
#endif
#ifdef OUTPUT
            SIO_idle (hTxSio);
#endif
            start = 1;
            continue;
        }
        else if (diff)
            LOG_printf (&trace, "Lock obtained");
#endif
			gecho++;

#ifdef OUTPUT
    errno = SIO_ctrl (hTxSio, PAF_SIO_CONTROL_SET_RATEX, (Arg) &ratex);
    if (errno) {
        LOG_printf (&trace, "Failed (0x%0x) to setRateX for hTxSio. Aborting", errno);
        TSK_sleep (10);
        return;
    }
#endif

        if (start) {
#ifdef INPUT
            for (i=0; i < sioAttrs.nbufs; i++) {
                errno = SIO_issue (hRxSio, rxPtr[i], bufSizeRx, 0);
                if (errno) {
                    LOG_printf (&trace, "RX start%i: SIO_issue error 0x%0x", i, errno);
                    break;
                }
            }
				gecho1++;
#endif
#if 1
            // kickstart output if not running
            for (i=0; i < sioAttrs.nbufs; i++) {
#ifdef OUTPUT
                errno = SIO_issue (hTxSio, txPtr[i], bufSizeTx, 0);
                if (errno) {
                    LOG_printf (&trace, "TX start: SIO_issue error 0x%0x", errno);
                    break;
                }
#endif

            }
#endif
            start = 0;
            frame = 0;
        }

        if (errno)
            continue;
		gecho2++;
#ifdef INPUT
        // wait for input buffer full
        nbytes = SIO_reclaim (hRxSio, &pRxBuf, NULL);

        if (nbytes != bufSizeRx) {
            errno = -nbytes;
            LOG_printf (&trace, "RX: reclaim = 0x%x (failed)", errno);
            continue;
        }
        tri1 = TSCL;
#endif
		gecho3++;
#ifdef OUTPUT
        // wait for output buffer empty
		nbytes = SIO_reclaim (hTxSio, &pTxBuf, NULL);
        if (nbytes != bufSizeTx) {
            errno = -nbytes;
            LOG_printf (&trace, "TX: reclaim = 0x%x (failed)", errno);
            continue;
        }
#endif
		tro1= TSCL;
#ifdef OUTPUT
        if (!pTxBuf && frame > 1) {
            errno = 1;
            LOG_printf (&trace, "Frame error", errno);
            continue;
        }

        if (!pTxBuf)
            pTxBuf = txPtr[frame];
#endif

// verify data in dsp matches transmitter
		//if(frame==0)
		//fail_flag_rx = bfm_check_rx(pRxBuf,&TestSrc[0]);


#if defined (INPUT) && defined (OUTPUT)
#if 1

        // process data
        errno = bufcpy ((int *)pRxBuf, (int *)pTxBuf, numChanRx, numChanTx, wrdSizeRx, wrdSizeTx, RATE_MULT_TX, BLOCK_SIZE);
        if (errno) {
            LOG_printf (&trace, "processData error 0x%0x", errno );
            continue;
        }
#endif
#endif
		//for(i=0;i<256;i++)
		//	*((int*)pTxBuf+i)= 0xffffffff;
#ifdef INPUT
        // release input buffer
        tii2 = TSCL;
        errno = SIO_issue (hRxSio, pRxBuf, bufSizeRx, NULL );

        if (errno) {
            LOG_printf (&trace, "Rx: Issue error 0x%0x", errno );
            ts1 = (int)((float)(tii2-tri1)/300000.0);
            continue;
        }
#endif

#ifdef OUTPUT
        // transmit output buffer
        errno = SIO_issue (hTxSio, pTxBuf, bufSizeTx, 0);

        if (errno) {
            LOG_printf (&trace, "Tx: Issue error 0x%0x", errno );
            continue;
        }
#endif
		ts2 = (int)((float)(tii2-tri1)/300000.0);
        frame++;

    } //while
#ifdef OUTPUT
        // wait for output buffer empty

		nbytes = SIO_reclaim (hTxSio, &pTxBuf, NULL);

        if (nbytes != bufSizeTx) {
            errno = -nbytes;
            LOG_printf (&trace, "TX: reclaim = 0x%x (failed)", errno);
        }
#endif
#ifdef OUTPUT
        // wait for output buffer empty

		nbytes = SIO_reclaim (hTxSio, &pTxBuf, NULL);

        if (nbytes != bufSizeTx) {
            errno = -nbytes;
            LOG_printf (&trace, "TX: reclaim = 0x%x (failed)", errno);
        }
#endif

} //testLoopback

// -----------------------------------------------------------------------------

static void printStatus (PAF_SIO_InputStatus *pStatus)
{
    LOG_printf( &trace, "Input Status" );
    LOG_printf( &trace, "--------------------" );

    LOG_printf( &trace,"\t%s", lockStates[pStatus->lock] );
    LOG_printf( &trace,"\t%s", audioStates[pStatus->nonaudio] );
    LOG_printf( &trace,"\t%s", emphasisStates[pStatus->emphasis] );
    LOG_printf( &trace,"\t%s", sampleRates[pStatus->sampleRateMeasured] );
    LOG_printf( &trace,"" );
    
} //PrintStatus

// -----------------------------------------------------------------------------
