
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Common SNG algorithm interface declarations
//


/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the SNG algorithm.
 */
#ifndef ISNG_
#define ISNG_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>

#include "icom.h"
#include "pafdec.h"
#include "audfil.h"

#include "ifil.h"
#include "fil.h"
#include "fil_tii.h"
#include "fil_tii_priv.h"

#include "paftyp.h"

/*
 *  ======== ISNG_Obj ========
 *  Every implementation of ISNG *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct ISNG_Obj {
    struct ISNG_Fxns *fxns;    /* function list: standard, public, private */
} ISNG_Obj;

/*
 *  ======== ISNG_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct ISNG_Obj *ISNG_Handle;

/*
 *  ======== ISNG_Status ========
 *  Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.
 */
typedef volatile struct ISNG_Status {
    Int size;
    XDAS_Int8 mode;      // disable, enable
    XDAS_Int8 channel;   // 0..N-1 (N)
    XDAS_Int8 command;   // 0x00: none
                         // 0x01: channel reset
                         // 0x02: channel advance
                         // 0x03: channel pause (sticky)
    XDAS_Int8 song;      // 0x00: nil
                         // 0x1n: noise:
                         // 0xm1: 0: satellite, 1: subwoofer
                         // 0xm2: 0: no ASP, 1: yes ASP [not implemented]
                         // 0xm8: 0: channel, 1: sequence
                         // 0x2n: tone:
                         // 0xm1: 0: satellite, 1: subwoofer
                         // 0xm2: 0: no ASP, 1: yes ASP [not implemented]
                         // 0xm8: 0: channel, 1: sequence
                         // 0x3n: multi-tone
                         // 0xm2: 0: no ASP, 1: yes ASP [not implemented]
	XDAS_Int8 unused2[4];
    XDAS_Int8 volume;    // volume (dB)
    XDAS_Int8 unused;
    XDAS_Int16 interval; // sequence interval (ms)
    XDAS_Int16 freq[2];  // frequency for tone, base for multi-tone (Hz)
                         // 0: satellite, 1: subwoofer
    XDAS_Int16 mult[2];  // frequency multiplier for multi-tone (Q8)
                         // 0: satellite, 1: subwoofer
    XDAS_Int16 phase[2]; // phase delay for multi-tone (ms)
                         // 0: satellite, 1: subwoofer
	XDAS_Int8 unused3[4]; //To make 8 byte alligned
	PAF_ChannelConfiguration channelConfigurationProgram;
} ISNG_Status;

/*
 *  ======== ISNG_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct ISNG_Config {
    Int size;
    XDAS_Int16 minimumFrameLength;
    XDAS_Int16 maximumFrameLength;
    XDAS_UInt32 seed; // could be common
    XDAS_UInt32 phaseSample; // could be common
    XDAS_UInt32 countSample; // could be common
    PAF_FilParam *pfilparam;
    XDAS_Int32 prevChannelConfig;	    			    	
} ISNG_Config;

/*
 *  ======== ISNG_Active ========
 *  Active structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm but which must maintain value
 *  only while the algorithm is running.
 */
typedef struct ISNG_Active {
    Int size;
    Int flag;
    PAF_DecodeStatus *pDecodeStatus;
} ISNG_Active;

/*
 *  ======== ISNG_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a SNG object.
 *
 *  Every implementation of ISNG *must* declare this structure as
 *  the first member of the implementation's parameter structure.
 */
typedef struct ISNG_Params {
    Int size;
    const ISNG_Status *pStatus;
    const ISNG_Config *pConfig;
    const ISNG_Active *pActive;
} ISNG_Params;

/*
 *  ======== ISNG_PARAMS ========
 *  Default instance creation parameters (defined in isng.c)
 */
extern const ISNG_Params ISNG_PARAMS;

/*
 *  ======== ISNG_Fxns ========
 *  All implementation's of SNG must declare and statically 
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is SNG_<vendor>_ISNG, where
 *  <vendor> is the vendor name.
 */

#define ISNG_FXNS_NOISEWHITE 0
#define ISNG_FXNS_NOISEPINK 5
#define ISNG_FXNS_N 8

typedef struct ISNG_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int         (*reset)(ISNG_Handle, ALG_Handle, PAF_DecodeControl *, PAF_DecodeStatus *);
    Int         (*info)(ISNG_Handle, ALG_Handle, PAF_DecodeControl *, PAF_DecodeStatus *);
    Int         (*decode)(ISNG_Handle, ALG_Handle, PAF_DecodeInStruct *, PAF_DecodeOutStruct *);
    /* private */
    Int         (*reset2)(ISNG_Handle, ALG_Handle);
    Int         (*command)(ISNG_Handle, ALG_Handle, PAF_DecodeInStruct *, PAF_DecodeOutStruct *);
    Int         (*sing[ISNG_FXNS_N])(ISNG_Handle, ALG_Handle, PAF_DecodeInStruct *, PAF_DecodeOutStruct *);
    
} ISNG_Fxns;

/*
 *  ======== ISNG_Cmd ========
 *  The Cmd enumeration defines the control commands for the SNG
 *  control method.
 */
typedef enum ISNG_Cmd {
    ISNG_NULL                   = ICOM_NULL,
    ISNG_GETSTATUSADDRESS1      = ICOM_GETSTATUSADDRESS1,
    ISNG_GETSTATUSADDRESS2      = ICOM_GETSTATUSADDRESS2,
    ISNG_GETSTATUS              = ICOM_GETSTATUS,
    ISNG_SETSTATUS              = ICOM_SETSTATUS,
    ISNG_MINSAMGEN,
    ISNG_MININFO,
    ISNG_MAXSAMGEN
} ISNG_Cmd;

#endif  /* ISNG_ */
