
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Standard SNG algorithm interface declarations
//
//

/*
 *  This header defines all types, constants, and functions used by 
 *  applications that use the SNG algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  the ability to incorporate multiple implementations of the SNG
 *  algorithm in a single application at the expense of some
 *  additional indirection.
 */
#ifndef SNG_
#define SNG_

#include <alg.h>
#include <isng.h>
#include <ialg.h>
#include <paf_alg.h>

/*
 *  ======== SNG_Handle ========
 *  SNG algorithm instance handle
 */
typedef struct ISNG_Obj *SNG_Handle;

/*
 *  ======== SNG_Params ========
 *  SNG algorithm instance creation parameters
 */
typedef struct ISNG_Params SNG_Params;

/*
 *  ======== SNG_PARAMS ========
 *  Default instance parameters
 */
#define SNG_PARAMS ISNG_PARAMS

/*
 *  ======== SNG_Status ========
 *  Status structure for getting SNG instance attributes
 */
typedef volatile struct ISNG_Status SNG_Status;

/*
 *  ======== SNG_Cmd ========
 *  This typedef defines the control commands SNG objects
 */
typedef ISNG_Cmd   SNG_Cmd;

/*
 * ===== control method commands =====
 */
#define SNG_NULL		ISNG_NULL
#define SNG_GETSTATUSADDRESS1	ISNG_GETSTATUSADDRESS1
#define SNG_GETSTATUSADDRESS2	ISNG_GETSTATUSADDRESS2
#define SNG_GETSTATUS		ISNG_GETSTATUS
#define SNG_SETSTATUS		ISNG_SETSTATUS
#define SNG_MINSAMGEN		ISNG_MINSAMGEN
#define SNG_MININFO		ISNG_MININFO
#define SNG_MAXSAMGEN		ISNG_MAXSAMGEN

/*
 *  ======== SNG_create ========
 *  Create an instance of a SNG object.
 */
static inline SNG_Handle SNG_create(const ISNG_Fxns *fxns, const SNG_Params *prms)
{
    return ((SNG_Handle)PAF_ALG_create((IALG_Fxns *)fxns, NULL, (IALG_Params *)prms,NULL,NULL));
}

/*
 *  ======== SNG_decode ========
 */
extern Int SNG_decode(SNG_Handle, ALG_Handle, PAF_DecodeInStruct *, PAF_DecodeOutStruct *);

/*
 *  ======== SNG_delete ========
 *  Delete a SNG instance object
 */
static inline Void SNG_delete(SNG_Handle handle)
{
    PAF_ALG_delete((ALG_Handle)handle);
}

/*
 *  ======== SNG_exit ========
 *  Module finalization
 */
extern Void SNG_exit(Void);

/*
 *  ======== SNG_info ========
 */
extern Int SNG_info(SNG_Handle, ALG_Handle, PAF_DecodeControl *, PAF_DecodeStatus *);

/*
 *  ======== SNG_init ========
 *  Module initialization
 */
extern Void SNG_init(Void);

/*
 *  ======== SNG_reset ========
 */
extern Int SNG_reset(SNG_Handle, ALG_Handle, PAF_DecodeControl *, PAF_DecodeStatus *);

#endif  /* SNG_ */
