/*
*  Copyright (c) 2004  2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (TI Devices). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <std.h>
#include <string.h>
#include <sys.h>
#include <sem.h>
#include <log.h>
#include <tsk.h>

#include <dsplink.h>
#include <dsplinkmsgq.h>
#include <failure.h>
#include <msgq.h>
#include <pool.h>
#include <platform.h>
#include <ringio.h>
#include <sma_pool.h>
#include <zcpy_mqt.h>

#include <acp.h>
#include <acp_mds.h>
#include <acp_mds_priv.h>
#include "afp_link.h"
#include <dri_ringio.h>
#include <dro_ringio.h>

SMAPOOL_Params SamplePoolParams = {SAMPLE_POOL_ID, TRUE};

static MSGQ_Obj msgQueues [NUM_MSG_QUEUES];

MSGQ_TransportObj transports [MAX_PROCESSORS] =
{
    MSGQ_NOTRANSPORT,
    MSGQ_NOTRANSPORT
};

MSGQ_Config MSGQ_config =
{
    msgQueues,
    transports,
    NUM_MSG_QUEUES,
    MAX_PROCESSORS,
    0,
    MSGQ_INVALIDMSGQ,
    POOL_INVALIDID
};

static POOL_Obj pools [NUM_POOLS] = {POOL_NOENTRY};

POOL_Config POOL_config = {pools, NUM_POOLS};

SMAPOOL_Params  smaPoolObj;
ZCPYMQT_Params  mqtParams;

volatile unsigned int DSPLINK_initFlag = 0xBABAC0C0;
Bool PA_DSPLINK_isInitialized = FALSE;

RingIO_Handle PA_DSPLINK_readerHandle = NULL;
RingIO_Handle PA_DSPLINK_writerHandle = NULL;

// When built using LINK Task mode for communication synchronization then
// grab reference for MQT ctrl. This is modified below to adjust
// the thread priority and stack.
#ifdef DSP_TSK_MODE
#include <dsplinkmqt.h>
extern DSPLINKMQT_Ctrl * DSPLINKMQT_ctrlPtr;
#endif
// -----------------------------------------------------------------------------

static ACP_Unit rxBuf[PA_LINK_MAX_CODE_LEN/sizeof(ACP_Unit)];
static ACP_Unit txBuf[PA_LINK_MAX_CODE_LEN/sizeof(ACP_Unit)];

void afpLinkTask (void)
{
    Int                          status = SYS_OK;
    MSGQ_Attrs                msgqAttrs = MSGQ_ATTRS;
    RingIO_Attrs            ringIoAttrs;
    RingIO_Attrs            rringIoAttrs;
    MSGQ_LocateAttrs    syncLocateAttrs;
    MSGQ_TransportObj         transport;
    POOL_Obj                    poolObj;
    Uint32                        flags;

    PA_LINK_Msg         *pMsg;
    AFP_LINK_Obj         afpLinkObj;
    ACP_Handle           acp;
    IALG_Status         *pStatus;

    // .........................................................................
    // Init ACP

    ACP_MDS_init ();
    acp = ACP_create (&ACP_MDS_IACP, &ACP_PARAMS);
    if (!acp)
        return;

    ((ALG_Handle)acp)->fxns->algControl((IALG_Handle)(acp),
                                        ACP_GETSTATUSADDRESS1,
                                        (IALG_Status *)&pStatus);

    acp->fxns->attach (acp,
                       ACP_SERIES_STD,
                       STD_BETA_UART,
                       pStatus);

    acp->fxns->attach (acp,
                       ACP_SERIES_STD,
                       STD_BETA_BETATABLE,
                       (IALG_Status *)((ACP_MDS_Obj *)acp)->config.betaTable[ACP_SERIES_STD]);

    acp->fxns->attach (acp,
                       ACP_SERIES_STD,
                       STD_BETA_PHITABLE,
                       (IALG_Status *)((ACP_MDS_Obj *)acp)->config.phiTable[ACP_SERIES_STD]);

    acp->fxns->attach (acp,
                       ACP_SERIES_STD,
                       STD_BETA_SIGMATABLE,
                       (IALG_Status *)((ACP_MDS_Obj *)acp)->config.sigmaTable[ACP_SERIES_STD]);

    // Set the semaphore to a known state.
    SEM_new (&afpLinkObj.notifySemObj, 0);

#ifdef PA_NO_BOOT
    // .........................................................................
    // General DSP/LINK Initialization

    // Wait for ARM signal before initializing communications
    while (DSPLINK_initFlag != 0xC0C0BABA)
        TSK_sleep (1);
    DSPLINK_init ();
    PA_DSPLINK_isInitialized = TRUE;
#endif

    // .........................................................................
    // POOL Initialization

    smaPoolObj.poolId        = 0;
    smaPoolObj.exactMatchReq = TRUE;
    poolObj.initFxn          = SMAPOOL_init;
    poolObj.fxns             = (POOL_Fxns *) &SMAPOOL_FXNS;
    poolObj.params           = &(smaPoolObj);
    poolObj.object           = NULL;

    status = POOL_open (0, &poolObj);
    if (status != SYS_OK) {
        SET_FAILURE_REASON (status);
        return;
    }

    // .........................................................................
    // RingIO Initialization
    //   Create and Open ring so it is always valid for GPP side
 
   ringIoAttrs.transportType  = RINGIO_TRANSPORT_GPP_DSP;
   ringIoAttrs.ctrlPoolId     = SAMPLE_POOL_ID;
   ringIoAttrs.dataPoolId     = SAMPLE_POOL_ID;
   ringIoAttrs.attrPoolId     = SAMPLE_POOL_ID;
   ringIoAttrs.lockPoolId     = SAMPLE_POOL_ID;
   ringIoAttrs.dataBufSize    = RINGIODATABUFSIZE;
   ringIoAttrs.footBufSize    = 0;
   ringIoAttrs.attrBufSize    = RINGIOATTRBUFSIZE;

   status = RingIO_create (ID_GPP, RING_IO_NAME, &ringIoAttrs);
   if (status != RINGIO_SUCCESS) {
       SET_FAILURE_REASON (status);
      return;
  }
//
//  //  Value of the flags indicates:
//  //     Cache coherence required for: Control structure   Data buffer                                       Attribute buffer
//   ///    Exact size requirement false.
   flags = (RINGIO_DATABUF_CACHEUSE | RINGIO_ATTRBUF_CACHEUSE | RINGIO_CONTROL_CACHEUSE);

   PA_DSPLINK_readerHandle = RingIO_open (RING_IO_NAME, RINGIO_MODE_READ, flags);
   if (!PA_DSPLINK_readerHandle)
      return;

 // .........................................................................
    // Reverse RingIO Initialization
    //   Create and Open ring so it is always valid for GPP side

    rringIoAttrs.transportType  = RINGIO_TRANSPORT_GPP_DSP;
    rringIoAttrs.ctrlPoolId     = SAMPLE_POOL_ID;
    rringIoAttrs.dataPoolId     = SAMPLE_POOL_ID;
    rringIoAttrs.attrPoolId     = SAMPLE_POOL_ID;
    rringIoAttrs.lockPoolId     = SAMPLE_POOL_ID;
    rringIoAttrs.dataBufSize    = RRINGIODATABUFSIZE;
    rringIoAttrs.footBufSize    = 0;
    rringIoAttrs.attrBufSize    = RRINGIOATTRBUFSIZE;

    status = RingIO_create (ID_GPP, RRING_IO_NAME, &rringIoAttrs);
    
    if (status != RINGIO_SUCCESS) {
        SET_FAILURE_REASON (status);
        return;
    }

    // Value of the flags indicates:
    //    Cache coherence required for: Control structure
    //                                  Data buffer
    //                                  Attribute buffer
    //    Exact size requirement false.
    flags = (RINGIO_DATABUF_CACHEUSE | RINGIO_ATTRBUF_CACHEUSE | RINGIO_CONTROL_CACHEUSE);

    PA_DSPLINK_writerHandle = RingIO_open (RRING_IO_NAME, RINGIO_MODE_WRITE, flags);
    if (!PA_DSPLINK_writerHandle)
        return;

    // .........................................................................
    // MSGQ Initialization

    // If needed adjust dynamically created thread with different priority and
    // stack. If it is non-zero then assume it was adjusted at LINK library
    // build time and don't modifiy.
#ifdef DSP_TSK_MODE
    if (DSPLINKMQT_ctrlPtr->config.arg1 == 0)
        DSPLINKMQT_ctrlPtr->config.arg1 = 9;

    if (DSPLINKMQT_ctrlPtr->config.arg2 == 0)
        DSPLINKMQT_ctrlPtr->config.arg2 = 0x800;
#endif

    mqtParams.poolId  = 0;
    transport.initFxn = SAMPLEMQT_init;          
    transport.fxns    = (MSGQ_TransportFxns *) &SAMPLEMQT_FXNS;
    transport.params  = &mqtParams;
    transport.object  = NULL;
    transport.procId  = ID_GPP;
    
    status = MSGQ_transportOpen (ID_GPP, &transport);
    if (status != SYS_OK) {
        SET_FAILURE_REASON (status);
        return;
    }

    // Fill in the attributes for this message queue.
    msgqAttrs.notifyHandle = &afpLinkObj.notifySemObj;
    msgqAttrs.pend         = (MSGQ_Pend) SEM_pendBinary;
    msgqAttrs.post         = (MSGQ_Post) SEM_postBinary;

    status = MSGQ_open (DSP_MSGQNAME, &afpLinkObj.localMsgq, &msgqAttrs);
    if (status == SYS_OK) {
        // Set the message queue that will receive any async. errors.
        MSGQ_setErrorHandler (afpLinkObj.localMsgq, SAMPLE_POOL_ID);

        // Synchronous
        status = SYS_ENOTFOUND;
        while ((status == SYS_ENOTFOUND) || (status == SYS_ENODEV)) {
            syncLocateAttrs.timeout = SYS_FOREVER;
            status = MSGQ_locate (GPP_MSGQNAME,
                                  &afpLinkObj.locatedMsgq,
                                  &syncLocateAttrs);
            if ((status == SYS_ENOTFOUND) || (status == SYS_ENODEV)) {
                TSK_sleep (1);
            }
            else if (status != SYS_OK) {
                SET_FAILURE_REASON (status);
                return;
            }
        }
    }
    else {
        SET_FAILURE_REASON (status);
        return;
    }

    // .........................................................................
    // Loop waiting for messages from GPP. When present process

    for (;;) {
        Int cnt, i, error, len, res_len;
        Int encap = 0;
        Int rxOffset = 0;
        Int txOffset = 0;
        Int rxIdx = 2;
        Int txIdx = 2;
        Int txStrip = 0;

        status = MSGQ_get (afpLinkObj.localMsgq, (MSGQ_Msg *) &pMsg, SYS_FOREVER);
        if (status != SYS_OK) {
            SET_FAILURE_REASON (status);
            TSK_exit ();
            return;
        }

        // total MSQG payload in bytes (not including first two for count)
        cnt = (pMsg->alfaCode[1] << 8) | pMsg->alfaCode[0];

        // copy, for now, payload to temp buffer -- helps align code with afp5.c
        memcpy (&rxBuf[rxIdx], &pMsg->alfaCode[2], cnt);

        // convert cnt to # of ACP_Units
        cnt /= sizeof (ACP_Unit);

        // if present, strip encapsulation and recompute count
        if ((rxBuf[rxIdx] & 0xFF00) == 0xC900) {
            cnt = rxBuf[rxIdx] & 0x00FF;
            rxIdx++;
            encap = 1;
        }
        else if (rxBuf[rxIdx] == 0xCD01) {
            cnt = rxBuf[rxIdx+1];
            rxIdx +=2;
            encap = 2;
        }

        // process one C-record at a time;
        while (cnt) {

            // check that message is really alpha code
            // if not then stop all processing since the length
            // computation is not meaningful and therefore we are not
            // able to skip to the next possible valid code
            if ((rxBuf[rxIdx] & 0xC000) != 0xC000) {
                txBuf[2] = 0xdead;
                txIdx=3;
                break;
            }

            // get length of this C-record and it's response length
            len = acp->fxns->length (acp, (ACP_Unit *) &rxBuf[rxIdx]);
            res_len = acp->fxns->htgnel (acp, (ACP_Unit *) &rxBuf[rxIdx]);

            // check that response length will fit in message            
            if (res_len > (PA_LINK_MAX_CODE_LEN/sizeof(ACP_Unit) - txIdx)) {
                txBuf[2] = 0xdead;
                txIdx=3;
                break;
            }

            // add encapuslation for this C-record (needed by ACP)
            if (len < 256) {
                rxBuf[--rxIdx] = 0xC900 + len;
                rxOffset = 1;
            }
            else {
                rxBuf[--rxIdx] = len;
                rxBuf[--rxIdx] = 0xCD01;
                rxOffset = 2;
            }

            error = acp->fxns->sequence (acp, &rxBuf[rxIdx], &txBuf[txIdx]);
            if (error) {
                if (res_len || encap) {
                    txBuf[txIdx++] = 0xDEAD;
                }
                cnt -= len;
                rxIdx += len + rxOffset;
                continue;
            }
            else {
                if ((txBuf[txIdx] & 0xFF00) == 0xC900)
                    txStrip = 1;
                else if (txBuf[txIdx] == 0xCD01)
                    txStrip = 2;
                for (i=0; i < res_len; ++i)
                    txBuf[txIdx+i] = txBuf[txIdx+i+txStrip];
            }

            cnt   -= len;
            rxIdx += len + rxOffset;
            txIdx += res_len;
        } //while (cnt)

        // If input is encapsulated then encapsulate the output
        if (encap) {
            if (txIdx < 258) {
                pMsg->alfaCode[2] = (txIdx - 2);
                pMsg->alfaCode[3] = 0xC9;
                txOffset = 1;
            }
            else {
                // TODO;
                txOffset = 2;
            }
        } //encap

        cnt = 2*(txIdx-2+txOffset);
        pMsg->alfaCode[0] = cnt & 0xFF;
        pMsg->alfaCode[1] = (cnt & 0xFF00) >> 8;
        memcpy (&pMsg->alfaCode[2+2*txOffset], &txBuf[2], 2*txIdx);

        status = MSGQ_put (afpLinkObj.locatedMsgq, (MSGQ_Msg) pMsg);
        if (status != SYS_OK) {
            SET_FAILURE_REASON (status);
            return;
        }

    } //for (;;)

    // .........................................................................
    // TODO: perform MSGQ cleanup on exit

    return;
} //afpLinkTask

#ifdef PA_NO_BOOT
// -----------------------------------------------------------------------------
// ISR which is executed to tell DSP, now it is time to rum DSPLINK_init
void AFP_LINK_initIsr (Ptr arg)
{
    volatile Uint32 * chipsig_clr = (Uint32 *) 0x1c14178u;

    (Void) arg ;
    /* Set the INIT Flag */
    DSPLINK_initFlag = 0xC0C0BABA;
    *chipsig_clr = 0x4;

} // AFP_LINK_initIsr
#endif
// -----------------------------------------------------------------------------
