
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Alpha File Processing via DSPLINK declarations
//
//
//


#ifndef AFP_LINK_H
#define AFP_LINK_H

#include <sem.h>

#include <msgq.h>
#include <sma_pool.h>
#include <zcpy_mqt.h>

typedef struct AFP_LINK_Obj {
    MSGQ_Queue localMsgq;
    MSGQ_Queue locatedMsgq;
    SEM_Obj    notifySemObj;
} AFP_LINK_Obj;

#define NUM_POOLS          1
#define NUM_MSG_QUEUES     1
#define NUM_MSG_POOLS      4
#define NUM_MSG_IN_POOL0   1
#define NUM_MSG_IN_POOL1   2
#define NUM_MSG_IN_POOL2   2
#define NUM_MSG_IN_POOL3   4

#define SAMPLEMQT_FXNS          ZCPYMQT_FXNS
#define SAMPLEMQT_init          ZCPYMQT_init
#define SAMPLEMQT_CTRLMSG_SIZE  ZCPYMQT_CTRLMSG_SIZE

#define SAMPLEPOOL_PARAMS       &SamplePoolParams
#define SAMPLEPOOL_FXNS         SMAPOOL_FXNS
#define SAMPLEPOOL_init         SMAPOOL_init

#define DSP_MSGQNAME        "DSPMSGQ1"
#define GPP_MSGQNAME        "GPPMSGQ1"
#define SAMPLE_POOL_ID      0

#define PA_LINK_MAX_CODE_LEN (1024 - sizeof (MSGQ_MsgHeader))

typedef struct PA_LINK_Msg {
    Uint32 reserved[2];                          // Transport specific
    Uint16 srcProcId;                            // Proc Id for the src message queue
    Uint16 poolId;                               // Id of the allocator that allocated the msg
    Uint16 size;                                 // Size of the allocated msg
    Uint16 dstId;                                // Destinaton message queue id
    Uint16 srcId;                                // Source message queue id
    Uint16 msgId;                                // User specified message id
    Uint8  alfaCode[PA_LINK_MAX_CODE_LEN];       // alpha code paylaod
} PA_LINK_Msg;

#define PA_LINK_MSG_SIZE  DSPLINK_ALIGN (sizeof (PA_LINK_Msg), DSPLINK_BUF_ALIGN)

#define RINGIODATABUFSIZE   0x3C000
#define RINGIOATTRBUFSIZE   0x600

#define RRINGIODATABUFSIZE   0x6000
#define RRINGIOATTRBUFSIZE   0x100

#endif //AFP_LINK_H
