#
# $Source: /cvsstl/ti/pa/f/s19/simda830/test.awk,v $
# $Revision: 1.2 $
#
# Awk script to parse output from PA I1 (or similar) "simda830" simulation
#
# Copyright 2008 Texas Instruments Incorporated.  All rights reserved.
# Texas Instruments Proprietary and Confidential.
# Use without a license from Texas Instruments is prohibited.
#
# $Log: test.awk,v $
# -----------------------------------------------------------------------------

# Sample usage:
#   awk -f test.awk < mcasp1_out.txt

# ............................................................................
# BEGIN runs once at beginning of records

BEGIN {
    aRun = 0;
    zeroRun = 0;
    frameStart = 0;
    frameCount = 0;
    zeroRunTrigger = 2;
    numSamples = 0;
    recordNum = 0;
    numChannels = 8;
    paFrameCount = 0;
}

# ------------------------------------------------------------------------------
# this routine runs for each record

{
    line = $1;
    recordNum += 1;

    # don't count blank lines, most likely these are at the end of the file
    if (line ~ /^$/) {
        next;
    }
    numSamples += 1;

    # on start counting zeroRun after threshold
    # if in zeroRun and encounter non-zero value then indicate end of run and reset
    if (line ~ /^00000000$/) {
        zeroRun += 1;
        if (zeroRun >= zeroRunTrigger) {
            frameCount = 0;
        }
    }
    else {
        if (zeroRun >= zeroRunTrigger) {
            printf ("zeroRun    = %d\n", zeroRun);
        }
        zeroRun = 0;
    }
    
    if (line ~ /^aaaaaaaa$/) {
        aRun += 1;
    }
    else if (aRun) {
        printf ("aRun       = %d\n", aRun);
        aRun = 0;
    }

    # if not in zeroRun or aRun then assume data
    if ((zeroRun < zeroRunTrigger) && !aRun) {        
        string = sprintf ("0x%s", line);
        tmp = strtonum (string);
        frameNum  = rshift (tmp,16);                        
        sampleNum = rshift (and (tmp, 0x00000F00), 8);
#        printf ("num = 0x%0x\tFrame = %d\tSample = 0x%0x\n", tmp, frameNum, sampleNum);

        # if beginning and first sample and frame are not zero then error and exit
        if (frameCount == 0) {
            if ((frameNum != 0) || (sampleNum != 0)) {
                printf ("Unexpected beginning of frame (%s)\t at line = %d\n", line, recordNum);
            }
        }
        else {
            # samples must be continuous mod the number of channels
            if (sampleNum != ((lastSampleNum + 1) % numChannels)) {
                printf ("Unexpected continued sample (%s)\t at line = %d\n", line, recordNum);
            }
            
            # frame counter increases once every numChannels
            if ((sampleNum && (frameNum != lastFrameNum)) ||
                (!sampleNum && (frameNum != lastFrameNum + 1))) {

                if ((line ~ /^00000000$/) && (lastLine ~ /^03ff0700$/)) {
                    printf ("frameCount = %d\tInput wrap around line = %d\n", paFrameCount, recordNum);
                    paFrameCount = 0;
                }
                else {
                    printf ("Unexpected continued frame (%s)\t at line = %d\n", line, recordNum);
                }
            }
            else if (frameNum != lastFrameNum) {
                paFrameCount += 1;
            }
        }
        frameCount += 1;
        lastFrameNum = frameNum;
        lastSampleNum = sampleNum;
        lastLine = line;
    }
}

# ------------------------------------------------------------------------------
# END runs after all records and we use it to print any summary results

END {
    if (!error) {
        if (aRun) {
            printf ("aRun    = %d\n", aRun);
        }

        if (zeroRun) {
            printf ("zeroRun      = %d\n", zeroRun);
        }
    
        if (frameCount) {
            printf ("frameCount = %d\n", paFrameCount);
        }
        printf ("numSamples = %d\n", numSamples);
    }
}

# ------------------------------------------------------------------------------
