/*
*  Copyright (c) 2004  2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (TI Devices). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// PA6 Listening Modes

#include <acptype.h>
#include <pafstd_a.h>
#include <pafsys_a.h>
#include <pafdec_a.h>
#include <pafvol_a.h>
//#include <thx_a.h>
#include "ad_a.h"
#include "asa_a.h"
#include "bc_a.h"
#include "bgc_a.h"
#include "tm_a.h"
#include "req_a.h"
#include "pl2x_a.h"
#include "neox_a.h"

#include <thxsys_a.h>

#define LM_THXCINEMA_S \
    wroteSYSListeningModeTHXCinema, \
    writeNEOXOperationalModeCinema,  \
    writeNEOXChannelCfgUnknown, \
    writePL2XOperationModeMovie,   \
    writePL2XChannelConfigurationOverrideUnknown, \
    writePL2XDEXBypassDisable,  \
    writePL2XDEXApplicationLevel1, \
    writePL2XDEXOperationModeEX, \
    writeADApplicationLevel1, \
    writeREQBypassDisable, \
    writeREQMaskSelectAll,  \
    writeASAOperationModeCinema, \
    writeTMOperationModeON, \
    execSTDReady

#define LM_THXEX_S \
    wroteSYSListeningModeTHXEX, \
    writePL2XDEXBypassDisable,  \
    writePL2XDEXApplicationLevel2, \
    writePL2XDEXOperationModeEX, \
    writePL2XOperationModeMovie,  \
    writePL2XChannelConfigurationOverrideUnknown, \
    writeNEOXOperationalModeCinema, \
    writeNEOXChannelCfgUnknown, \
    writeADApplicationLevel0,  \
    writeREQBypassDisable, \
    writeREQMaskSelectAll,  \
    writeASAOperationModeEX,  \
    writeTMOperationModeON, \
    execSTDReady

#define LM_THXMUSIC_S \
    wroteSYSListeningModeTHXMusic, \
    writeNEOXOperationalModeMusic,  \
    writeNEOXChannelCfgUnknown, \
    writePL2XOperationModeMusic, \
    writePL2XChannelConfigurationOverrideUnknown, \
    writePL2XDEXBypassDisable,    \
    writePL2XDEXApplicationLevel1, \
    writePL2XDEXOperationModeEX, \
    writeADApplicationLevel1,  \
    writeREQBypassDisable, \
    writeREQMaskSelect0,   \
    writeASAOperationModeMusic,  \
    writeTMOperationModeMusic, \
    execSTDReady

#define LM_THXGAMES_S \
    wroteSYSListeningModeTHXGames, \
    writeNEOXOperationalModeCinema,  \
    writeNEOXChannelCfgUnknown, \
    writePL2XOperationModeMovie, \
    writePL2XChannelConfigurationOverrideSurround2_0, \
    writePL2XDEXBypassDisable,    \
    writePL2XDEXApplicationLevel1, \
    writePL2XDEXOperationModeEX, \
    writeADApplicationLevel0,  \
    writeREQBypassDisable, \
    writeREQMaskSelect0,   \
    writeASAOperationModeGames,  \
    writeTMOperationModeON, \
    execSTDReady

#pragma DATA_SECTION(lm_thxcinema_s0, ".none")
const ACP_Unit lm_thxcinema_s0[] = {
    0xc900 + 0 - 1,
    LM_THXCINEMA_S,
};

const ACP_Unit lm_thxcinema_s[] = {
    0xc900 + sizeof(lm_thxcinema_s0)/2 - 1,
    LM_THXCINEMA_S,
};

#pragma DATA_SECTION(lm_thxex_s0, ".none")
const ACP_Unit lm_thxex_s0[] = {
    0xc900 + 0 - 1,
    LM_THXEX_S,
};

const ACP_Unit lm_thxex_s[] = {
    0xc900 + sizeof(lm_thxex_s0)/2 - 1,
    LM_THXEX_S,
};

#pragma DATA_SECTION(lm_thxmusic_s0, ".none")
const ACP_Unit lm_thxmusic_s0[] = {
    0xc900 + 0 - 1,
    LM_THXMUSIC_S,
};

const ACP_Unit lm_thxmusic_s[] = {
    0xc900 + sizeof(lm_thxmusic_s0)/2 - 1,
    LM_THXMUSIC_S,
};

#pragma DATA_SECTION(lm_thxgames_s0, ".none")
const ACP_Unit lm_thxgames_s0[] = {
    0xc900 + 0 - 1,
    LM_THXGAMES_S,
};

const ACP_Unit lm_thxgames_s[] = {
    0xc900 + sizeof(lm_thxgames_s0)/2 - 1,
    LM_THXGAMES_S,
};

#define LM_UNKNOWN_S \
    wroteSYSListeningModeUnknown, \
    execSTDReady

#pragma DATA_SECTION(lm_unknown_s0, ".none")
const ACP_Unit lm_unknown_s0[] = {
    0xc900 + 0 - 1,
    LM_UNKNOWN_S,
};

const ACP_Unit lm_unknown_s[] = {
    0xc900 + sizeof(lm_unknown_s0)/2 - 1,
    LM_UNKNOWN_S,
};

#define LM_STANDARD_S \
    writeDECASPGearControlAll, \
    writeSYSRecreationModeAuto, \
    writeVOLImplementationInternal, /* Okay only for now. --Kurt */ \
    wroteSYSListeningModeStandard,  \
    writePL2XOperationModeMovie,  \
    writePL2XChannelConfigurationOverrideUnknown, \
    writeNEOXOperationalModeCinema, \
    writeNEOXChannelCfgUnknown, \
    writePL2XDEXApplicationLevel2, \
    writeADApplicationLevel0,  \
    writeREQBypassEnable, \
    writeREQMaskSelect0,  \
    writeASAOperationModeOFF,  \
    writeTMOperationModeOFF, \
    execSTDReady

#pragma DATA_SECTION(lm_standard_s0, ".none")
const ACP_Unit lm_standard_s0[] = {
    0xc900 + 0 - 1,
    LM_STANDARD_S,
};

const ACP_Unit lm_standard_s[] = {
    0xc900 + sizeof(lm_standard_s0)/2 - 1,
    LM_STANDARD_S,
};

#define LM_PURE_S \
    writeDECASPGearControlNil, \
    writeSYSRecreationModeDont,writeDECChannelConfigurationRequestUnknown, \
    writeVOLImplementationInactive, \
    wroteSYSListeningModePure, \
    execSTDReady

#pragma DATA_SECTION(lm_pure_s0, ".none")
const ACP_Unit lm_pure_s0[] = {
    0xc900 + 0 - 1,
    LM_PURE_S,
};

const ACP_Unit lm_pure_s[] = {
    0xc900 + sizeof(lm_pure_s0)/2 - 1,
    LM_PURE_S,
};

