
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Standard Bass Management Algoritm interface declarations
//

/*
 *  This header defines all types, constants, and functions used by 
 *  applications that use the Bass Management Algoritm.
 *
 *  Applications that use this interface enjoy type safety and
 *  the ability to incorporate multiple implementations of the BM
 *  Algorithm in a single application at the expense of some
 *  additional indirection.
 */
#ifndef BM_
#define BM_

#include <alg.h>
#include <ibm.h>
#include <ialg.h>
#include <paf_alg.h>

#include "paftyp.h"

/*
 *  ======== BM_Handle ========
 *  Bass Management Algoritm instance handle
 */
typedef struct IBM_Obj *BM_Handle;

/*
 *  ======== BM_Params ========
 *  Bass Management Algoritm instance creation parameters
 */
typedef struct IBM_Params BM_Params;

/*
 *  ======== BM_PARAMS ========
 *  Default instance parameters
 */
#define BM_PARAMS IBM_PARAMS

/*
 *  ======== BM_Status ========
 *  Status structure for getting BM instance attributes
 */
typedef volatile struct IBM_Status BM_Status;

/*
 *  ======== BM_Cmd ========
 *  This typedef defines the control commands BM objects
 */
typedef IBM_Cmd   BM_Cmd;

/*
 * ===== control method commands =====
 */
#define BM_NULL IBM_NULL
#define BM_GETSTATUSADDRESS1 IBM_GETSTATUSADDRESS1
#define BM_GETSTATUSADDRESS2 IBM_GETSTATUSADDRESS2

#define BM_GETSTATUSFILHPMAIN IBM_GETSTATUSFILHPMAIN
#define BM_GETSTATUSFILHPCNTR IBM_GETSTATUSFILHPCNTR
#define BM_GETSTATUSFILHPSURR IBM_GETSTATUSFILHPSURR
#ifdef PAF_BACK
#define BM_GETSTATUSFILHPBACK IBM_GETSTATUSFILHPBACK
#define BM_GETSTATUSFILLPREARLEFT IBM_GETSTATUSFILLPREARLEFT
#define BM_GETSTATUSFILLPREARRGHT IBM_GETSTATUSFILLPREARRGHT
#endif /* PAF_BACK */
#define BM_GETSTATUSFILLPMAIN IBM_GETSTATUSFILLPMAIN
#define BM_GETSTATUSFILLPCNTR IBM_GETSTATUSFILLPCNTR
#define BM_GETSTATUSFILLPSURR IBM_GETSTATUSFILLPSURR
#define BM_GETSTATUSFILLPBACK IBM_GETSTATUSFILLPBACK
#define BM_GETSTATUSFILLPLFE IBM_GETSTATUSFILLPLFE
#define BM_GETSTATUSFILLPSUBW IBM_GETSTATUSFILLPSUBW

#define BM_GETSTATUS IBM_GETSTATUS
#define BM_SETSTATUS IBM_SETSTATUS

/*
 *  ======== BM_create ========
 *  Create an instance of a BM object.
 */
static inline BM_Handle BM_create(const IBM_Fxns *fxns, const BM_Params *prms)
{
    return ((BM_Handle)PAF_ALG_create((IALG_Fxns *)fxns, NULL, (IALG_Params *)prms,NULL,NULL));
}

/*
 *  ======== BM_delete ========
 *  Delete a BM instance object
 */
static inline Void BM_delete(BM_Handle handle)
{
    PAF_ALG_delete((ALG_Handle)handle);
}

/*
 *  ======== BM_apply ========
 */
extern Int BM_apply(BM_Handle, PAF_AudioFrame *);

/*
 *  ======== BM_reset ========
 */
extern Int BM_reset(BM_Handle, PAF_AudioFrame *);

/*
 *  ======== BM_exit ========
 *  Module finalization
 */
extern Void BM_exit(Void);

/*
 *  ======== BM_init ========
 *  Module initialization
 */
extern Void BM_init(Void);

#endif  /* BM_ */
