
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Bass Management algorithm filter declarations
//


#include "fil_datatype.h"
#include <ibm.h>

#ifndef BM_FILCOEF_1_
#define BM_FILCOEF_1_

typedef struct BMFilCoefs_SP {
    Uint type;
    Uint sampRate;
    float *coefs[1];
} BMFilCoefs_SP;

/* BM local channel definitions: */

#ifdef PAF_BACK
#define BM_LPREARLEFT       0
#define BM_LPREARRGHT       0
#endif /* PAF_BACK */

#define BM_LPMAIN0          0
#ifdef PAF_RSUB
#define BM_LPMAIN1          1
#endif /* PAF_RSUB */

#define BM_LPCNTR           0

#define BM_LPSURR0          0
#ifdef PAF_RSUB
#define BM_LPSURR1          1
#endif /* PAF_RSUB */

#define BM_LPBACK0          0
#ifdef PAF_RSUB
#define BM_LPBACK1          1
#endif /* PAF_RSUB */

#define BM_LPWIDE0          0
#ifdef PAF_RSUB
#define BM_LPWIDE1          1
#endif /* PAF_RSUB */

#define BM_LPHIGH0          0
#ifdef PAF_RSUB
#define BM_LPHIGH1          1
#endif /* PAF_RSUB */

#define BM_LPLFE0           0
#ifdef PAF_RSUB
#define BM_LPLFE1           1
#endif /* PAF_RSUB */

/* Filter types: */

#define BM_FILTYPE2TAP1CH   ( (0u<<31)  /* not multi-sample-rate */ \
                              +(1<<29)  /* single-precision */ \
                              +(0<<23)  /* IR */ \
                              +(0<<19)  /* IIR */ \
                              +(1<<18)  /* uni-coefficient */ \
							  +(1<<15)  /* mx1 filters */ \
                              +(2<<5)   /* 2-tap */ \
                              +(1<<0)   /* 1-channel */ )

#define BM_FILTYPE2TAP2CH   ( (0u<<31)  /* not multi-sample-rate */ \
                              +(1<<29)  /* single-precision */ \
                              +(0<<23)  /* IR */ \
                              +(0<<19)  /* IIR */ \
                              +(1<<18)  /* uni-coefficient */ \
							  +(1<<15)  /* mx1 filters */ \
                              +(2<<5)   /* 2-tap */ \
                              +(2<<0)   /* 2-channel */ )

#define BM_FILTYPE4TAP1CH   ( (0u<<31)  /* not multi-sample-rate */ \
                              +(1<<29)  /* single-precision */ \
                              +(1<<23)  /* IR */ \
                              +(1<<19)  /* IIR */ \
                              +(1<<18)  /* uni-coefficient */ \
							  +(1<<15)  /* mx1 filters */ \
                              +(4<<5)   /* 4-tap */ \
                              +(1<<0)   /* 1-channel */ )

#define BM_FILTYPE4TAP2CH   ( (0u<<31)  /* not multi-sample-rate */ \
                              +(1<<29)  /* single-precision */ \
                              +(1<<23)  /* IR */ \
                              +(1<<19)  /* IIR */ \
                              +(1<<18)  /* uni-coefficient */ \
							  +(1<<15)  /* mx1 filters */ \
                              +(4<<5)   /* 4-tap */ \
                              +(2<<0)   /* 2-channel */ )

#define BM_FILTYPEHPMAIN            BM_FILTYPE2TAP2CH   /* Filter type */
#define BM_FILTYPEHPCNTR            BM_FILTYPE2TAP1CH
#define BM_FILTYPEHPSURR            BM_FILTYPE2TAP2CH
#ifdef PAF_BACK
#define BM_FILTYPEHPBACK            BM_FILTYPE2TAP2CH
#define BM_FILTYPEHPWIDE            BM_FILTYPE2TAP2CH
#define BM_FILTYPEHPHIGH            BM_FILTYPE2TAP2CH
#define BM_FILTYPELP2TAPREARLEFT    BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPREARLEFT    BM_FILTYPE4TAP1CH
#define BM_FILTYPELP2TAPREARRGHT    BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPREARRGHT    BM_FILTYPE4TAP1CH
#endif /* PAF_BACK */

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPMAIN        BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPMAIN        BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPMAIN        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPMAIN        BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#define BM_FILTYPELP2TAPCNTR        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPCNTR        BM_FILTYPE4TAP1CH

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPSURR        BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPSURR        BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPSURR        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPSURR        BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPBACK        BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPBACK        BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPBACK        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPBACK        BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPWIDE        BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPWIDE        BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPWIDE        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPWIDE        BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPHIGH        BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPHIGH        BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPHIGH        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPHIGH        BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPLFE         BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPLFE         BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPLFE         BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPLFE         BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#ifdef PAF_RSUB
#define BM_FILTYPELP2TAPSUBW        BM_FILTYPE2TAP2CH
#define BM_FILTYPELP4TAPSUBW        BM_FILTYPE4TAP2CH
#else
#define BM_FILTYPELP2TAPSUBW        BM_FILTYPE2TAP1CH
#define BM_FILTYPELP4TAPSUBW        BM_FILTYPE4TAP1CH
#endif /* PAF_RSUB */

#endif  /* BM_FILCOEF_1_ */
