
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


/*
 *  Vendor specific (MDS) interface header for
 *  Bass Management Algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  and minimal overhead at the expense of being tied to a
 *  particular BM implementation.
 *
 *  This header only contains declarations that are specific
 *  to this implementation.  Thus, applications that do not
 *  want to be tied to a particular implementation should never
 *  include this header (i.e., it should never directly
 *  reference anything defined in this header.)
 */
#ifndef BM_MDS_
#define BM_MDS_

#include <ialg.h>
#include <ibm.h>
#include <bm_filcoef.h>

/*
 *  ======== BM_MDS_exit ========
 *  Required module finalization function
 */
#define BM_MDS_exit COM_TII_exit

/*
 *  ======== BM_MDS_init ========
 *  Required module initialization function
 */
#define BM_MDS_init COM_TII_init

/*
 *  ======== BM_MDS_IALG ========
 *  MDS's implementation of BM's IALG interface
 */
extern IALG_Fxns BM_MDS_IALG; 

/*
 *  ======== BM_MDS_IBM ========
 *  MDS's implementation of BM's IBM interface
 */
extern const IBM_Fxns BM_MDS_IBM; 


/*
 *  ======== Vendor specific methods  ========
 *  The remainder of this file illustrates how a vendor can
 *  extend an interface with custom operations.
 *
 *  The operations below simply provide a type safe interface 
 *  for the creation, deletion, and application of MDS's Bass
 *  Management Algorithm. However, other implementation specific
 *  operations can also be added.
 */

/*
 *  ======== BM_MDS_Handle ========
 */
typedef struct BM_MDS_Obj *BM_MDS_Handle;

/*
 *  ======== BM_MDS_Params ========
 *  We don't add any new parameters to the standard ones defined by IBM.
 */
typedef IBM_Params BM_MDS_Params;

/*
 *  ======== BM_MDS_Status ========
 *  We don't add any new status to the standard one defined by IBM.
 */
typedef IBM_Status BM_MDS_Status;

/*
 *  ======== BM_MDS_Config ========
 *  We add new fields not in IBM:
 */
typedef struct BM_MDS_Config 
{
    Int size;
    /* Additional new fields for BM2: */
    float *pCoefsHpMain;    /* BM high pass filter coefs (converted) */
    float *pCoefsHpCntr;
    float *pCoefsHpSurr;
    float *pCoefsHpBack;
    float *pCoefsLpMain;    /* BM low pass filter coefs (converted) */
    float *pCoefsLpCntr;
    float *pCoefsLpSurr;
    float *pCoefsLpBack;
    float *pCoefsLpLFE;
    float *pCoefsLpSubw;
    BMFilCoefs_SP bmCoefsDefaultHpMain;
    BMFilCoefs_SP bmCoefsDefaultHpCntr;
    BMFilCoefs_SP bmCoefsDefaultHpSurr;
    BMFilCoefs_SP bmCoefsDefaultHpBack;
    BMFilCoefs_SP bmCoefsDefaultLpRearLeft;
    BMFilCoefs_SP bmCoefsDefaultLpRearRght;
    BMFilCoefs_SP bmCoefsDefaultLpMain;
    BMFilCoefs_SP bmCoefsDefaultLpCntr;
    BMFilCoefs_SP bmCoefsDefaultLpSurr;
    BMFilCoefs_SP bmCoefsDefaultLpBack;
    BMFilCoefs_SP bmCoefsDefaultLpLFE;
    BMFilCoefs_SP bmCoefsDefaultLpSubw;
    /*  insert additional parms here so legacy-in ROM routines can be used. */
    float *pCoefsHpWide;
    float *pCoefsHpHigh;
    float *pCoefsLpWide;
    float *pCoefsLpHigh;
    BMFilCoefs_SP bmCoefsDefaultHpWide;
    BMFilCoefs_SP bmCoefsDefaultHpHigh;
    BMFilCoefs_SP bmCoefsDefaultLpWide;
    BMFilCoefs_SP bmCoefsDefaultLpHigh;
    Int blksize; // max sampleCount for which scrach buffers are created
    /*
    **  Pointers to buffers in scrach memory immediately following *pScrach
    */
    PAF_AudioData *filBuffer; 
    PAF_AudioData *sclBuffer0;
    PAF_AudioData *sclBuffer1;
    PAF_AudioData *sumBuffer0;
  #ifdef PAF_RSUB
    PAF_AudioData *sumBuffer1;
  #endif
}BM_MDS_Config;

/*
 *  ======== BM_MDS_Active ========
 *  We don't add any new active to the standard one defined by IBM.
 */
 
typedef IBM_Active BM_MDS_Active;

/*
 *  ======== BM_MDS_Scrach ========
 *  We don't add any new active to the standard one defined by IBM.
 */
 
typedef IBM_Scrach BM_MDS_Scrach;

/*
 *  ======== BM_MDS_PARAMS ========
 *  Define our default parameters.
 */
#define BM_MDS_PARAMS   IBM_PARAMS

/*
 *  ======== BM_MDS_create ========
 *  Create a BM_MDS instance object.
 */
extern BM_MDS_Handle BM_MDS_create(const BM_MDS_Params *params);

/*
 *  ======== BM_MDS_delete ========
 *  Delete a BM_MDS instance object.
 */
#define BM_MDS_delete COM_TII_delete

#endif  /* BM_MDS_ */
