
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// Common Bass Management Algoritm interface declarations

/*
 *  This header defines all types, constants, and functions shared by all
 *  implementations of the Bass Management Algoritm.
 */
#ifndef IBM_
#define IBM_

#include <ialg.h>
#include  <std.h>
#include <xdas.h>
#include <alg.h> /* for SIO */

#include "fil.h"
#include "icom.h"
#include "paftyp.h"

#include <bm_filcoef.h>	/* keep here after BM defines above */

/*
**  BLKSIZE specifies the default size (in samples) of audio buffers used for
**  filtering and redirection.  If BM is asked to operate on an audio frame
**  having a sampleCount greater than BLKSIZE, it refuses.
**
**  BM uses an initialization parameter to set BLKSIZE.  If that value is 
**  not provided, or is ridiculous, the default value specified here is used.
*/
#define BLKSIZE 256

#define BMMODE_DISABLE      0
#define BMMODE_LPF2NDORDER  1
#define BMMODE_LPF4THORDER  2
#define BMMODE_LPFAUTO      3

#define BMBYPASS_DISABLE    0
#define BMBYPASS_ENABLE     1

/*
 *  ======== IBM_Obj ========
 *  Every implementation of IBM *must* declare this structure as
 *  the first member of the implementation's object.
 */
typedef struct IBM_Obj {
    struct IBM_Fxns *fxns;    /* function list: standard, public, private */
} IBM_Obj;

/*
 *  ======== IBM_Handle ========
 *  This type is a pointer to an implementation's instance object.
 */
typedef struct IBM_Obj *IBM_Handle;

/*
 *  ======== IBM_Status ========
 *  Status structure defines the parameters that can be changed or read
 *  during real-time operation of the algorithm.
 */
typedef volatile struct IBM_Status {
    Int         size;

    XDAS_Int8   mode;
    XDAS_Int8   ocAuto;
    XDAS_Int8   inactionReason;  /* why BM isn't operating */
    XDAS_Int8   docLast;             /* ocSelect OCNumber remembered */

    XDAS_UInt32 ocSelect;
    XDAS_UInt32 ocStatus;

	/* MID 1776: invert channel to be treated as Small for bass redir */
    XDAS_Int8   overrideBassCollectionMask; 
    XDAS_Int8   frStatus;       /* filter (sample) rate status */
    XDAS_Int8   bypass;
    XDAS_Int8   LFEVolume;      /* LFE trim for bass summing */

    XDAS_Int8   modeLast;       /* mode history/change detector */
    XDAS_Int8   modeStatus;     /* indicates 2nd/4th order, useful if BMModeAuto */    
    XDAS_Int8   ccrSat8Last;	/* ChannelConfigurationRequest.part.sat on last setup */
    XDAS_Int8   ccrSub8Last;	/* channelConfigurationRequest.part.sub on last setup */

    XDAS_Int16  fcMain;         /* satellite filter cutoff controls */
    XDAS_Int16  fcCntr;
    XDAS_Int16  fcSurr;
    XDAS_Int16  fcBack;

    XDAS_Int16  fcStatusMain;   /* satellite filter cutoff status */
    XDAS_Int16  fcStatusCntr;
    XDAS_Int16  fcStatusSurr;
    XDAS_Int16  fcStatusBack;
                                /* (late additions:) */
    XDAS_Int16  fcLFE;          /* LFE/Subw filter cutoff controls */
    XDAS_Int16  fcSubw;
    XDAS_Int16  fcStatusLFE;    /* LFE/Subw filter cutoff status */
    XDAS_Int16  fcStatusSubw;

    XDAS_Int8   MainVolume;     /* Main trim for bass summing */
    XDAS_Int8   CntrVolume;     /* Cntr trim for bass summing */
    XDAS_Int8   SurrVolume;     /* Surr trim for bass summing */
    XDAS_Int8   BackVolume;     /* Back trim for bass summing */

    XDAS_Int16  fcWide;         /* fcWide: filter cutoff control (Hz) */
    XDAS_Int16  fcHigh;         /* fcHigh: filter cutoff control (Hz) */

    XDAS_Int16  fcStatusWide;   /* fcStatusWide: filter cutoff status (Hz) */
    XDAS_Int16  fcStatusHigh;   /* fcStatusHigh: filter cutoff status (Hz) */

    XDAS_Int8   WideVolume;     /* Wide trim for bass summing */
    XDAS_Int8   HighVolume;     /* High trim for bass summing */
    XDAS_Int8   ccrExt8Last;    /* ChannelConfigurationRequest.part.ext on last setup */
    XDAS_Int8   options;        /* bit 0 ON => use legacy samsiz */
} IBM_Status;

/*
 *  ======== IBM_Config ========
 *  Config structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm.
 */
typedef struct IBM_Config 
{
    Int size;
} IBM_Config;

/*
 *  ======== IBM_Scrach ========
 *  Scrach structure defines the parameters that cannot be changed or read
 *  during real-time operation of the algorithm and which must maintain value
 *  while the algorithm is running during a single invocation.
 */

typedef struct IBM_Scrach
{
    Int size;
    PAF_AudioFrame *pFrame;
    PAF_AudioFrame frame;
    PAF_ChannelConfigurationMaskTable maskTable;

    //  filBuffer[] ends up here
    //  sclBuffer0[] ends up here
    //  sclBuffer1[] ends up here
    //  sumBuffer0[] ends up here
    //  sumBuffer1[] ends up here

} IBM_Scrach;

/*
 *  ======== IBM_Params ========
 *  This structure defines the parameters necessary to create an
 *  instance of a BM object.
 *
 *  Every implementation of IBM *must* declare this structure as
 *  the first member of the implementation's parameter structure.
 */

typedef struct IBM_Active {
    Int size;
    Int flag;
    IFIL_Fxns *ifil_Fxns;
}IBM_Active;

extern const IBM_Active IBM_ACTIVE;

/*
**  Redefine the IBM_Params structure to include new fields at the end.
**  With the new fields at the end, legacy routines which use this structure
**  can be head/tail patched so that the ROM version uses the legacy fields.
*/ 
typedef struct IBM_Params 
{
    Int size;
    const IBM_Status  *pStatus;
    const IBM_Config  *pConfig;
    const IBM_Active  *pActive;
    const IBM_Scrach  *pScrach;
    const IFIL_Params *pParamsHpMain;
    const IFIL_Params *pParamsHpCntr;
    const IFIL_Params *pParamsHpSurr;
#ifdef PAF_BACK
    const IFIL_Params *pParamsHpBack;
    const IFIL_Params *pParamsLpRearLeft;
    const IFIL_Params *pParamsLpRearRght;
#endif /* PAF_BACK */
    const IFIL_Params *pParamsLpMain;
    const IFIL_Params *pParamsLpCntr;
    const IFIL_Params *pParamsLpSurr;
    const IFIL_Params *pParamsLpBack;
    const IFIL_Params *pParamsLpLFE;
    const IFIL_Params *pParamsLpSubw;

    ///////////  new fields follow (for MID 1918 and 3346) /////////////////
    Int blksize;  // the blksize for which the BM instance was created.
    const IFIL_Params *pParamsHpWide; // Added for MID 3346
    const IFIL_Params *pParamsHpHigh; // Added for MID 3346
    const IFIL_Params *pParamsLpWide; // Added for MID 3346
    const IFIL_Params *pParamsLpHigh; // Added for MID 3346
} IBM_Params;
/*
 *  ======== IBM_PARAMS ========
 *  Instance creation parameters (defined in ibm???.c)
 */
extern const IBM_Params IBM_PARAMS_256;
extern const IBM_Params IBM_PARAMS_512;

/*
**  The following two constants are offsets into BM_COLLECTION.maxSample[]
*/
#define PAF_SUMBUFFER0 (PAF_MAXNUMCHAN + 0)
#define PAF_SUMBUFFER1 (PAF_MAXNUMCHAN + 1)

/*
**  BM_COLLECTION is passed between the realapply functions so realapply
**  can be broken down into multiple functions, to make it more easily patched.
*/
typedef struct bmCollection
{
    PAF_AudioFrame *pScrachFrame;
    PAF_ChannelMask satMask;
    PAF_ChannelMask satSmallMask;
    PAF_ChannelMask satSmallMaskRestore;
    PAF_ChannelMask subMask;
    XDAS_Int16      ccrSat;
    PAF_AudioSize   sumBuffer0Siz;
#ifdef PAF_RSUB
    PAF_AudioSize   sumBuffer1Siz;
#endif
    /*
    **  Use maxSample[] to keep track of the maximum potential value
    **  of the sample in a channel.
    **  Obtain initially from the audioFrame's samsiz[].
    **  Adjust maxSample[] as bass is redirected by BM.
    **  At end, adjust samsiz[] from maxSample[].
    **  It's computationally easier to track maxSample[] inside BM2
    **  than to track samsiz[].
    */
    PAF_AudioData   maxSample[PAF_MAXNUMCHAN + 2];
    Int bUseLegacySamsiz; // computed as status.options & 0x01	
}
BM_COLLECTION;

/*
 *  ======== IBM_Fxns ========
 *  All implementation's of BM must declare and statically 
 *  initialize a constant variable of this type.
 *
 *  By convention the name of the variable is BM_<vendor>_IBM, where
 *  <vendor> is the vendor name.
 */
typedef struct IBM_Fxns {
    /* public */
    IALG_Fxns   ialg;
    Int         (*reset)(IBM_Handle, PAF_AudioFrame *);
    Int         (*apply)(IBM_Handle, PAF_AudioFrame *);
    /* private */
    Int         (*setup)(IBM_Handle, PAF_AudioFrame *, Int);
    Int         (*realapply)(IBM_Handle, PAF_AudioFrame *);
    Int         (*active)(IBM_Handle, PAF_AudioFrame *);
    void        (*CtoLR)(IBM_Handle, PAF_AudioFrame *, BM_COLLECTION * );
    void        (*LBRear)(IBM_Handle, PAF_AudioFrame *, BM_COLLECTION * );
    void        (*HPFSatellites)(IBM_Handle, PAF_AudioFrame *, BM_COLLECTION * );
    PAF_ChannelMask (*setMinimumStreamConfig)( PAF_AudioFrame *pAudioFrame, PAF_ChannelMask mCHANS );
    void        (*sumSmallSatellites)(IBM_Handle, PAF_AudioFrame *, BM_COLLECTION * );
    void        (*redirectBassToSatellites)(IBM_Handle, PAF_AudioFrame *, BM_COLLECTION * );
    void        (*largeToSubwoofer)(IBM_Handle, PAF_AudioFrame *, BM_COLLECTION * );
} IBM_Fxns;

/*
 *  ======== IBM_Cmd ========
 *  The Cmd enumeration defines the control commands for the BM
 *  control method.
 */
typedef enum IBM_Cmd {
    IBM_NULL                    = ICOM_NULL,
    IBM_GETSTATUSADDRESS1       = ICOM_GETSTATUSADDRESS1,
    IBM_GETSTATUSADDRESS2       = ICOM_GETSTATUSADDRESS2,
    IBM_GETSTATUS               = ICOM_GETSTATUS,
    IBM_SETSTATUS               = ICOM_SETSTATUS,
    IBM_GETSTATUSFILHPMAIN,
    IBM_GETSTATUSFILHPCNTR,
    IBM_GETSTATUSFILHPSURR,
#ifdef PAF_BACK
    IBM_GETSTATUSFILHPBACK,
    IBM_GETSTATUSFILLPREARLEFT,
    IBM_GETSTATUSFILLPREARRGHT,
#endif /* PAF_BACK */
    IBM_GETSTATUSFILLPMAIN,
    IBM_GETSTATUSFILLPCNTR,
    IBM_GETSTATUSFILLPSURR,
    IBM_GETSTATUSFILLPBACK,
    IBM_GETSTATUSFILLPLFE,
    IBM_GETSTATUSFILLPSUBW
} IBM_Cmd;

#endif  /* IBM_ */
