
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// GEQ interface declarations
//
//
//

/*
 *  This header defines all types, constants, and functions used by 
 *  applications that use the ASP Example Demonstration algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  the ability to incorporate multiple implementations of the GEQ
 *  algorithm in a single application at the expense of some
 *  additional indirection.
 */
#ifndef GEQ_
#define GEQ_

#include <alg.h>
#include <ialg.h>
#include <paf_alg.h>
#include <igeq.h>

#include "paftyp.h"


/*
 *  ======== GEQ_Handle ========
 *  ASP Example Demonstration algorithm instance handle
 */
typedef struct IGEQ_Obj *GEQ_Handle;

/*
 *  ======== GEQ_Params ========
 *  ASP Example Demonstration algorithm instance creation parameters
 */
typedef struct IGEQ_Params GEQ_Params;

/*
 *  ======== GEQ_PARAMS ========
 *  Default instance parameters
 */
#define GEQ_PARAMS IGEQ_PARAMS

/*
 *  ======== GEQ_Status ========
 *  Status structure for getting GEQ instance attributes
 */
typedef volatile struct IGEQ_Status GEQ_Status;

/*
 *  ======== GEQ_Cmd ========
 *  This typedef defines the control commands GEQ objects
 */
typedef IGEQ_Cmd   GEQ_Cmd;

/*
 * ===== control method commands =====
 */
#define GEQ_NULL IGEQ_NULL
#define GEQ_GETSTATUSADDRESS1 IGEQ_GETSTATUSADDRESS1
#define GEQ_GETSTATUSADDRESS2 IGEQ_GETSTATUSADDRESS2
#define GEQ_GETSTATUS IGEQ_GETSTATUS
#define GEQ_SETSTATUS IGEQ_SETSTATUS

/*
 *  ======== GEQ_create ========
 *  Create an instance of a GEQ object.
 */
static inline GEQ_Handle GEQ_create(const IGEQ_Fxns *fxns, const GEQ_Params *prms)
{
    return ((GEQ_Handle)PAF_ALG_create((IALG_Fxns *)fxns, NULL, (IALG_Params *)prms,NULL,NULL));
}

/*
 *  ======== GEQ_delete ========
 *  Delete a GEQ instance object
 */
static inline Void GEQ_delete(GEQ_Handle handle)
{
    PAF_ALG_delete((ALG_Handle)handle);
}

/*
 *  ======== GEQ_apply ========
 */
extern Int GEQ_apply(GEQ_Handle, PAF_AudioFrame *);

/*
 *  ======== GEQ_reset ========
 */
extern Int GEQ_reset(GEQ_Handle, PAF_AudioFrame *);

/*
 *  ======== GEQ_exit ========
 *  Module finalization
 */
extern Void GEQ_exit(Void);

/*
 *  ======== GEQ_init ========
 *  Module initialization
 */
extern Void GEQ_init(Void);

#endif  /* GEQ_ */
