
/*
*  Copyright (c) 2004 – 2014 Texas Instruments Incorporated
*  All rights reserved not granted herein.
*
*  Limited License. 
*
*  Texas Instruments Incorporated grants a world-wide, royalty-free,
*  non-exclusive license under copyrights and patents it now or hereafter 
*  owns or controls to make, have made, use, import, offer to sell and 
*  sell ("Utilize") this software subject to the terms herein. With respect
*  to the foregoing patent license, such license is granted solely to the 
*  extent that any such patent is necessary to Utilize the software alone. 
*  The patent license shall not apply to any combinations which include 
*  this software, other than combinations with devices manufactured 
*  by or for TI (“TI Devices”). No hardware patent is licensed hereunder.
*
*  Redistributions must preserve existing copyright notices and reproduce 
*  this license (including the above copyright notice and the disclaimer 
*  and (if applicable) source code license limitations below) in the 
*  documentation and/or other materials provided with the distribution
*
*  Redistribution and use in binary form, without modification, are permitted 
*  provided that the following conditions are met:
*
* 	No reverse engineering, decompilation, or disassembly of this software
*	is permitted with respect to any software provided in binary form.
*
* 	any redistribution and use are licensed by TI for use only with TI Devices.
*
* 	Nothing shall obligate TI to provide you with source code for the software 
*	licensed and provided to you in object code.
*
*  If software source code is provided to you, modification and redistribution 
*  of the source code are permitted provided that the following conditions are met:
*
*	any redistribution and use of the source code, including any resulting 
*	derivative works, are licensed by TI for use only with TI Devices.
*
*	any redistribution and use of any object code compiled from the source code 
*	and any resulting derivative works, are licensed by TI for use only 
*	with TI Devices.
*
*  Neither the name of Texas Instruments Incorporated nor the names of its 
*  suppliers may be used to endorse or promote products derived from 
*  this software without specific prior written permission.
*
*  DISCLAIMER.
*
*	THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" 
*	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
*	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
*	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS 
*	BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
*	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
*	OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
*	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
*	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
*	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
*	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
//
// GEQ interface declarations
//
//
//

/*
 *  Vendor specific (TII) interface header for
 *  ASP Example Demonstration algorithm.
 *
 *  Applications that use this interface enjoy type safety and
 *  and minimal overhead at the expense of being tied to a
 *  particular GEQ implementation.
 *
 *  This header only contains declarations that are specific
 *  to this implementation.  Thus, applications that do not
 *  want to be tied to a particular implementation should never
 *  include this header (i.e., it should never directly
 *  reference anything defined in this header.)
 */
#ifndef GEQ_TII_
#define GEQ_TII_

#include <ialg.h>

#include <igeq.h>
#include <icom.h>

/*
 *  ======== GEQ_TII_exit ========
 *  Required module finalization function
 */
#define GEQ_TII_exit COM_TII_exit

/*
 *  ======== GEQ_TII_init ========
 *  Required module initialization function
 */
#define GEQ_TII_init COM_TII_init

/*
 *  ======== GEQ_TII_IALG ========
 *  TII's implementation of GEQ's IALG interface
 */
extern IALG_Fxns GEQ_TII_IALG; 

/*
 *  ======== GEQ_TII_IGEQ ========
 *  TII's implementation of GEQ's IGEQ interface
 */
extern const IGEQ_Fxns GEQ_TII_IGEQ; 


/*
 *  ======== Vendor specific methods  ========
 *  The remainder of this file illustrates how a vendor can
 *  extend an interface with custom operations.
 *
 *  The operations below simply provide a type safe interface 
 *  for the creation, deletion, and application of TII's
 *  ASP Example Demonstration algorithm. However, other
 *  implementation specific operations can also be added.
 */

/*
 *  ======== GEQ_TII_Handle ========
 */
typedef struct GEQ_TII_Obj *GEQ_TII_Handle;

/*
 *  ======== GEQ_TII_Params ========
 *  We don't add any new parameters to the standard ones defined by IGEQ.
 */
typedef IGEQ_Params GEQ_TII_Params;

/*
 *  ======== GEQ_TII_Status ========
 *  We don't add any new status to the standard one defined by IGEQ.
 */
typedef IGEQ_Status GEQ_TII_Status;

/*
 *  ======== GEQ_TII_Config ========
 *  We don't add any new config to the standard one defined by IGEQ.
 */
typedef IGEQ_Config GEQ_TII_Config;

/*
 *  ======== GEQ_TII_PARAMS ========
 *  Define our default parameters.
 */
#define GEQ_TII_PARAMS   IGEQ_PARAMS

/*
 *  ======== GEQ_TII_create ========
 *  Create a GEQ_TII instance object.
 */
extern GEQ_TII_Handle GEQ_TII_create(const GEQ_TII_Params *params);

/*
 *  ======== GEQ_TII_delete ========
 *  Delete a GEQ_TII instance object.
 */
#define GEQ_TII_delete COM_TII_delete

#endif  /* GEQ_TII_ */
